@@ -36,7 +36,7 @@
 #include "lib/util/blocking.h"
 #include "lib/util/become_daemon.h"
 
-#include "common/version.h"
+#include "version.h"
 #include "ctdb_private.h"
 #include "ctdb_client.h"
 
@@ -928,44 +928,6 @@ static int ctdb_clientpid_destructor(struct ctdb_client_pid_list *client_pid)
 	return 0;
 }
 
-static int get_new_client_id(struct reqid_context *idr,
-			     struct ctdb_client *client,
-			     uint32_t *out)
-{
-	uint32_t client_id;
-
-	client_id = reqid_new(idr, client);
-	/*
-	 * Some places in the code (e.g. ctdb_control_db_attach(),
-	 * ctdb_control_db_detach()) assign a special meaning to
-	 * client_id 0.  The assumption is that if client_id is 0 then
-	 * the control has come from another daemon.  Therefore, we
-	 * should never return client_id == 0.
-	 */
-	if (client_id == 0) {
-		/*
-		 * Don't leak ID 0.  This is safe because the ID keeps
-		 * increasing.  A test will be added to ensure that
-		 * this doesn't change.
-		 */
-		reqid_remove(idr, 0);
-
-		client_id = reqid_new(idr, client);
-	}
-
-	if (client_id == REQID_INVALID) {
-		return EINVAL;
-	}
-
-	if (client_id == 0) {
-		/* Every other ID must have been used and we can't use 0 */
-		reqid_remove(idr, 0);
-		return EINVAL;
-	}
-
-	*out = client_id;
-	return 0;
-}
 
 static void ctdb_accept_client(struct tevent_context *ev,
 			       struct tevent_fd *fde, uint16_t flags,
@@ -1009,15 +971,7 @@ static void ctdb_accept_client(struct tevent_context *ev,
 
 	client->ctdb = ctdb;
 	client->fd = fd;
-
-	ret = get_new_client_id(ctdb->idr, client, &client->client_id);
-	if (ret != 0) {
-		DBG_ERR("Unable to get client ID (%d)\n", ret);
-		close(fd);
-		talloc_free(client);
-		return;
-	}
-
+	client->client_id = reqid_new(ctdb->idr, client);
 	client->pid = peer_pid;
 
 	client_pid = talloc(client, struct ctdb_client_pid_list);
@@ -1251,18 +1205,21 @@ static void ctdb_initialise_vnn_map(struct ctdb_context *ctdb)
 
 static void ctdb_set_my_pnn(struct ctdb_context *ctdb)
 {
+	int nodeid;
+
 	if (ctdb->address == NULL) {
 		ctdb_fatal(ctdb,
 			   "Can not determine PNN - node address is not set\n");
 	}
 
-	ctdb->pnn = ctdb_ip_to_pnn(ctdb, ctdb->address);
-	if (ctdb->pnn == CTDB_UNKNOWN_PNN) {
+	nodeid = ctdb_ip_to_nodeid(ctdb, ctdb->address);
+	if (nodeid == -1) {
 		ctdb_fatal(ctdb,
-			   "Can not determine PNN - unknown node address\n");
+			   "Can not determine PNN - node address not found in node list\n");
 	}
 
-	D_NOTICE("PNN is %u\n", ctdb->pnn);
+	ctdb->pnn = ctdb->nodes[nodeid]->pnn;
+	DEBUG(DEBUG_NOTICE, ("PNN is %u\n", ctdb->pnn));
 }
 
 /*
@@ -1273,14 +1230,14 @@ int ctdb_start_daemon(struct ctdb_context *ctdb, bool do_fork)
 	int res, ret = -1;
 	struct tevent_fd *fde;
 
-	become_daemon(do_fork, !do_fork, !do_fork);
+	become_daemon(do_fork, !do_fork, false);
 
 	ignore_signal(SIGPIPE);
 	ignore_signal(SIGUSR1);
 
 	ctdb->ctdbd_pid = getpid();
 	DEBUG(DEBUG_ERR, ("Starting CTDBD (Version %s) as PID: %u\n",
-			  ctdb_version_string, ctdb->ctdbd_pid));
+			  SAMBA_VERSION_STRING, ctdb->ctdbd_pid));
 	ctdb_create_pidfile(ctdb);
 
 	/* create a unix domain stream socket to listen to */
