@@ -956,12 +956,12 @@ static int delete_tdb_record(struct ctdb_context *ctdb, struct ctdb_db_context *
 	data.dptr = &rec->data[rec->keylen];
 
 	if (ctdb_lmaster(ctdb, &key) == ctdb->pnn) {
-		DEBUG(DEBUG_INFO,(__location__ " Called delete on record where we are lmaster\n"));
+		DBG_INFO("Called delete on record where we are lmaster\n");
 		return -1;
 	}
 
 	if (data.dsize != sizeof(struct ctdb_ltdb_header)) {
-		DEBUG(DEBUG_ERR,(__location__ " Bad record size\n"));
+		DBG_ERR("Bad record size\n");
 		return -1;
 	}
 
@@ -969,6 +969,7 @@ static int delete_tdb_record(struct ctdb_context *ctdb, struct ctdb_db_context *
 
 	/* use a non-blocking lock */
 	if (tdb_chainlock_nonblock(ctdb_db->ltdb->tdb, key) != 0) {
+		DBG_INFO("Failed to get non-blocking chain lock\n");
 		return -1;
 	}
 
@@ -981,10 +982,10 @@ static int delete_tdb_record(struct ctdb_context *ctdb, struct ctdb_db_context *
 	if (data2.dsize < sizeof(struct ctdb_ltdb_header)) {
 		if (tdb_lock_nonblock(ctdb_db->ltdb->tdb, -1, F_WRLCK) == 0) {
 			if (tdb_delete(ctdb_db->ltdb->tdb, key) != 0) {
-				DEBUG(DEBUG_CRIT,(__location__ " Failed to delete corrupt record\n"));
+				DBG_ERR("Failed to delete corrupt record\n");
 			}
 			tdb_unlock(ctdb_db->ltdb->tdb, -1, F_WRLCK);
-			DEBUG(DEBUG_CRIT,(__location__ " Deleted corrupt record\n"));
+			DBG_ERR("Deleted corrupt record\n");
 		}
 		tdb_chainunlock(ctdb_db->ltdb->tdb, key);
 		free(data2.dptr);
@@ -995,8 +996,9 @@ static int delete_tdb_record(struct ctdb_context *ctdb, struct ctdb_db_context *
 
 	if (hdr2->rsn > hdr->rsn) {
 		tdb_chainunlock(ctdb_db->ltdb->tdb, key);
-		DEBUG(DEBUG_INFO,(__location__ " Skipping record with rsn=%llu - called with rsn=%llu\n",
-			 (unsigned long long)hdr2->rsn, (unsigned long long)hdr->rsn));
+		DBG_INFO("Skipping record with rsn=%llu - called with rsn=%llu\n",
+			 (unsigned long long)hdr2->rsn,
+			 (unsigned long long)hdr->rsn);
 		free(data2.dptr);
 		return -1;
 	}
@@ -1004,26 +1006,27 @@ static int delete_tdb_record(struct ctdb_context *ctdb, struct ctdb_db_context *
 	/* do not allow deleting record that have readonly flags set. */
 	if (hdr->flags & CTDB_REC_RO_FLAGS) {
 		tdb_chainunlock(ctdb_db->ltdb->tdb, key);
-		DEBUG(DEBUG_INFO,(__location__ " Skipping record with readonly flags set\n"));
+		DBG_INFO("Skipping record with readonly flags set\n");
 		free(data2.dptr);
 		return -1;
 	}
 	if (hdr2->flags & CTDB_REC_RO_FLAGS) {
 		tdb_chainunlock(ctdb_db->ltdb->tdb, key);
-		DEBUG(DEBUG_INFO,(__location__ " Skipping record with readonly flags set\n"));
+		DBG_INFO("Skipping record with readonly flags set locally\n");
 		free(data2.dptr);
 		return -1;
 	}
 
 	if (hdr2->dmaster == ctdb->pnn) {
 		tdb_chainunlock(ctdb_db->ltdb->tdb, key);
-		DEBUG(DEBUG_INFO,(__location__ " Attempted delete record where we are the dmaster\n"));
+		DBG_INFO("Attempted delete record where we are the dmaster\n");
 		free(data2.dptr);
 		return -1;
 	}
 
 	if (tdb_lock_nonblock(ctdb_db->ltdb->tdb, -1, F_WRLCK) != 0) {
 		tdb_chainunlock(ctdb_db->ltdb->tdb, key);
+		DBG_INFO("Failed to get non-blocking freelist lock\n");
 		free(data2.dptr);
 		return -1;
 	}
@@ -1031,7 +1034,7 @@ static int delete_tdb_record(struct ctdb_context *ctdb, struct ctdb_db_context *
 	if (tdb_delete(ctdb_db->ltdb->tdb, key) != 0) {
 		tdb_unlock(ctdb_db->ltdb->tdb, -1, F_WRLCK);
 		tdb_chainunlock(ctdb_db->ltdb->tdb, key);
-		DEBUG(DEBUG_INFO,(__location__ " Failed to delete record\n"));
+		DBG_INFO("Failed to delete record\n");
 		free(data2.dptr);
 		return -1;
 	}
@@ -1415,57 +1418,12 @@ int32_t ctdb_control_set_recmaster(struct ctdb_context *ctdb, uint32_t opcode, T
 	return 0;
 }
 
-void ctdb_node_become_inactive(struct ctdb_context *ctdb)
-{
-	struct ctdb_db_context *ctdb_db;
-
-	D_WARNING("Making node INACTIVE\n");
-
-	/*
-	 * Do not service database calls - reset generation to invalid
-	 * so this node ignores any REQ/REPLY CALL/DMASTER
-	 */
-	ctdb->vnn_map->generation = INVALID_GENERATION;
-	for (ctdb_db = ctdb->db_list; ctdb_db != NULL; ctdb_db = ctdb_db->next) {
-		ctdb_db->generation = INVALID_GENERATION;
-	}
-
-	/*
-	 * Although this bypasses the control, the only thing missing
-	 * is the deferred drop of all public IPs, which isn't
-	 * necessary because they are dropped below
-	 */
-	if (ctdb->recovery_mode != CTDB_RECOVERY_ACTIVE) {
-		D_NOTICE("Recovery mode set to ACTIVE\n");
-		ctdb->recovery_mode = CTDB_RECOVERY_ACTIVE;
-	}
-
-	/*
-	 * Initiate database freeze - this will be scheduled for
-	 * immediate execution and will be in progress long before the
-	 * calling control returns
-	 */
-	ctdb_daemon_send_control(ctdb,
-				 ctdb->pnn,
-				 0,
-				 CTDB_CONTROL_FREEZE,
-				 0,
-				 CTDB_CTRL_FLAG_NOREPLY,
-				 tdb_null,
-				 NULL,
-				 NULL);
-
-	D_NOTICE("Dropping all public IP addresses\n");
-	ctdb_release_all_ips(ctdb);
-}
 
 int32_t ctdb_control_stop_node(struct ctdb_context *ctdb)
 {
 	DEBUG(DEBUG_ERR, ("Stopping node\n"));
 	ctdb->nodes[ctdb->pnn]->flags |= NODE_FLAGS_STOPPED;
 
-	ctdb_node_become_inactive(ctdb);
-
 	return 0;
 }
 
