@@ -24,7 +24,7 @@ EOF
 
 . "${TEST_SCRIPTS_DIR}/integration.bash"
 
-ctdb_test_init "$@"
+ctdb_test_init
 
 set -e
 
@@ -32,14 +32,15 @@ cluster_is_healthy
 
 try_command_on_node -v 0 "$CTDB listnodes"
 
-num_nodes=$(wc -l <"$outfile")
+num_nodes=$(echo "$out" | wc -l)
 
 # Each line should look like an IP address.
 ipv4_pat='[[:digit:]]+\.[[:digit:]]+\.[[:digit:]]+\.[[:digit:]]+'
 ipv6_pat='[[:xdigit:]]+:[[:xdigit:]:]+[[:xdigit:]]+'
 sanity_check_output \
     2 \
-    "^${ipv4_pat}|${ipv6_pat}\$"
+    "^${ipv4_pat}|${ipv6_pat}\$" \
+    "$out"
 
 out_0="$out"
 
@@ -54,7 +55,7 @@ while [ $n -lt $num_nodes ] ; do
     else
 	echo "DIFFERs from node 0:"
 	echo "$out"
-	testfailures=1
+	exit 1
     fi
     n=$(($n + 1))
 done
