@@ -26,29 +26,29 @@ EOF
 
 . "${TEST_SCRIPTS_DIR}/integration.bash"
 
-ctdb_test_init "$@"
+ctdb_test_init
 
 set -e
 
 cluster_is_healthy
 
 cmd="$CTDB isnotrecmaster || true"
-try_command_on_node -v all "$cmd"
+try_command_on_node all "$cmd"
+echo "Output of \"$cmd\":"
+echo "$out"
 
-num_all_lines=$(wc -l <"$outfile")
-num_rm_lines=$(grep -Fc 'this node is the recmaster' "$outfile") || true
-num_not_rm_lines=$(grep -Fc 'this node is not the recmaster' "$outfile") || true
+num_all_lines=$(echo "$out" |  wc -l)
+num_rm_lines=$(echo "$out" | fgrep -c 'this node is the recmaster') || true
+num_not_rm_lines=$(echo "$out" | fgrep -c 'this node is not the recmaster') || true
 
 if [ $num_rm_lines -eq 1 ] ; then
     echo "OK, there is only 1 recmaster"
 else
-    echo "BAD, there are ${num_rm_lines} nodes claiming to be the recmaster"
-    testfailures=1
+    die "BAD, there are ${num_rm_lines} nodes claiming to be the recmaster"
 fi
 
 if [ $(($num_all_lines - $num_not_rm_lines)) -eq 1 ] ; then
     echo "OK, all the other nodes claim not to be the recmaster"
 else
-    echo "BAD, there are only ${num_not_rm_lines} nodes claiming not to be the recmaster"
-    testfailures=1
+    die "BAD, there are only ${num_not_rm_lines} notrecmaster nodes"
 fi
