@@ -9,23 +9,20 @@ EOF
 
 . "${TEST_SCRIPTS_DIR}/integration.bash"
 
-ctdb_test_init "$@"
+ctdb_test_init
 
 set -e
 
 cluster_is_healthy
 
-# Reset configuration
-ctdb_restart_when_done
-
 testdb="rec_test.tdb"
 
 echo "Getting list of nodes..."
 try_command_on_node -v any "onnode -pq all ctdb pnn | grep '^[0-9][0-9]*$'"
 
-first=$(sed -n -e '1p' "$outfile")
-second=$(sed -n -e '2p' "$outfile")
-notfirst=$(tail -n +2 "$outfile")
+first=$(echo "$out" | sed -n -e '1p')
+second=$(echo "$out" | sed -n -e '2p')
+notfirst=$(echo "$out" | tail -n +2)
 
 echo "Create/wipe test database ${testdb}"
 try_command_on_node $first $CTDB attach "$testdb"
@@ -49,7 +46,7 @@ database_has_zero_records ()
 	local n
 	for n in $notfirst ; do
 		try_command_on_node $n $CTDB cattdb "$testdb"
-		if grep -q '^key(' "$outfile" ; then
+		if echo "$out" | grep -q '^key(' ; then
 			return 1
 		fi
 	done
@@ -57,11 +54,18 @@ database_has_zero_records ()
 	return 0
 }
 
-echo "Trigger a recovery"
-try_command_on_node "$second" $CTDB recover
+echo "Get vacuum interval"
+try_command_on_node -v $second $CTDB getvar VacuumInterval
+vacuum_interval="${out#* = }"
 
-echo "Checking that database has 0 records"
-database_has_zero_records
+echo "Wait until vacuuming deletes the record on active nodes"
+# Why 4?  Steps are:
+# 1. Original node processes delete queue, asks lmaster to fetch
+# 2. lmaster recoverd fetches
+# 3. lmaster processes delete queue
+# If vacuuming is just missed then need an extra interval
+t=$((vacuum_interval * 4))
+wait_until "${t}/10" database_has_zero_records
 
 echo "Continue node ${first}"
 try_command_on_node $first $CTDB continue
@@ -70,7 +74,7 @@ wait_until_node_has_status $first notstopped
 echo "Get database contents"
 try_command_on_node -v $first $CTDB catdb "$testdb"
 
-if grep -q '^key(' "$outfile" ; then
+if echo "$out" | grep -q '^key(' ; then
 	echo "BAD: Deleted record has been resurrected"
 	exit 1
 fi
