@@ -21,79 +21,76 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "includes.h"
+#include "replace.h"
 #include "system/filesys.h"
 #include "system/locale.h"
-#if HAVE_SYSTEMD
+#if defined(HAVE_LIBSYSTEMD_DAEMON) || defined(HAVE_LIBSYSTEMD)
 #include <systemd/sd-daemon.h>
 #endif
 
+#include "close_low_fd.h"
+#include "debug.h"
+
+#include "become_daemon.h"
+
 /*******************************************************************
  Close the low 3 fd's and open dev/null in their place.
 ********************************************************************/
 
-_PUBLIC_ void close_low_fds(bool stdin_too, bool stdout_too, bool stderr_too)
+void close_low_fds(bool stdin_too, bool stdout_too, bool stderr_too)
 {
-#ifndef VALGRIND
-	int fd;
-	int i;
 
-	if (stdin_too)
-		close(0);
-	if (stdout_too)
-		close(1);
-
-	if (stderr_too)
-		close(2);
-
-	/* try and use up these file descriptors, so silly
-		library routines writing to stdout etc won't cause havoc */
-	for (i=0;i<3;i++) {
-		if (i == 0 && !stdin_too)
-			continue;
-		if (i == 1 && !stdout_too)
-			continue;
-		if (i == 2 && !stderr_too)
-			continue;
-
-		fd = open("/dev/null",O_RDWR,0);
-		if (fd < 0)
-			fd = open("/dev/null",O_WRONLY,0);
-		if (fd < 0) {
-			DEBUG(0,("Can't open /dev/null\n"));
-			return;
+	if (stdin_too) {
+		int ret = close_low_fd(0);
+		if (ret != 0) {
+			DBG_ERR("close_low_fd(0) failed: %s\n", strerror(ret));
 		}
-		if (fd != i) {
-			DEBUG(0,("Didn't get file descriptor %d\n",i));
-			close(fd);
-			return;
+	}
+	if (stdout_too) {
+		int ret = close_low_fd(1);
+		if (ret != 0) {
+			DBG_ERR("close_low_fd(1) failed: %s\n", strerror(ret));
+		}
+	}
+	if (stderr_too) {
+		int ret = close_low_fd(2);
+		if (ret != 0) {
+			DBG_ERR("close_low_fd(2) failed: %s\n", strerror(ret));
 		}
 	}
-#endif
 }
 
 /****************************************************************************
  Become a daemon, discarding the controlling terminal.
 ****************************************************************************/
 
-_PUBLIC_ void become_daemon(bool do_fork, bool no_process_group, bool log_stdout)
+void become_daemon(bool do_fork, bool no_session, bool log_stdout)
 {
 	pid_t newpid;
 	if (do_fork) {
 		newpid = fork();
+		if (newpid == -1) {
+			exit_daemon("Fork failed", errno);
+		}
 		if (newpid) {
-#if HAVE_SYSTEMD
-			sd_notifyf(0, "READY=0\nSTATUS=Starting process...\nMAINPID=%lu", (unsigned long) newpid);
-#endif /* HAVE_SYSTEMD */
 			_exit(0);
 		}
+#if defined(HAVE_LIBSYSTEMD_DAEMON) || defined(HAVE_LIBSYSTEMD)
+	} else {
+		sd_notify(0, "STATUS=Starting process...");
+#endif
 	}
 
 	/* detach from the terminal */
 #ifdef HAVE_SETSID
-	if (!no_process_group) setsid();
+	if (!no_session) {
+		int ret = setsid();
+		if (ret == -1) {
+			exit_daemon("Failed to create session", errno);
+		}
+	}
 #elif defined(TIOCNOTTY)
-	if (!no_process_group) {
+	if (!no_session) {
 		int i = open("/dev/tty", O_RDWR, 0);
 		if (i != -1) {
 			ioctl(i, (int) TIOCNOTTY, (char *)0);
@@ -109,9 +106,9 @@ _PUBLIC_ void become_daemon(bool do_fork, bool no_process_group, bool log_stdout
 	close_low_fds(do_fork, !log_stdout, false);
 }
 
-_PUBLIC_ void exit_daemon(const char *msg, int error)
+void exit_daemon(const char *msg, int error)
 {
-#ifdef HAVE_SYSTEMD
+#if defined(HAVE_LIBSYSTEMD_DAEMON) || defined(HAVE_LIBSYSTEMD)
 	if (msg == NULL) {
 		msg = strerror(error);
 	}
@@ -121,17 +118,31 @@ _PUBLIC_ void exit_daemon(const char *msg, int error)
 				  msg,
 				  error);
 #endif
-	DEBUG(0, ("STATUS=daemon failed to start: %s, error code %d\n", msg, error));
+	DBG_ERR("daemon failed to start: %s, error code %d\n",
+		msg, error);
 	exit(1);
 }
 
-_PUBLIC_ void daemon_ready(const char *daemon)
+void daemon_ready(const char *daemon)
 {
 	if (daemon == NULL) {
 		daemon = "Samba";
 	}
-#ifdef HAVE_SYSTEMD
-	sd_notifyf(0, "READY=1\nSTATUS=%s: ready to serve connections...", daemon);
+#if defined(HAVE_LIBSYSTEMD_DAEMON) || defined(HAVE_LIBSYSTEMD)
+	sd_notifyf(0, "READY=1\nSTATUS=%s: ready to serve connections...",
+		   daemon);
 #endif
-	DEBUG(0, ("STATUS=daemon '%s' finished starting up and ready to serve connections", daemon));
+	DBG_ERR("daemon '%s' finished starting up and ready to serve "
+		"connections\n", daemon);
+}
+
+void daemon_status(const char *daemon, const char *msg)
+{
+	if (daemon == NULL) {
+		daemon = "Samba";
+	}
+#if defined(HAVE_LIBSYSTEMD_DAEMON) || defined(HAVE_LIBSYSTEMD)
+	sd_notifyf(0, "STATUS=%s: %s", daemon, msg);
+#endif
+	DBG_ERR("daemon '%s' : %s\n", daemon, msg);
 }
