@@ -15,6 +15,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+from __future__ import print_function
 """Tests for samba.dcerpc.dnsserver"""
 
 import os
@@ -28,13 +29,14 @@ from samba.tests import RpcInterfaceTestCase, env_get_var_value
 from samba.netcmd.dns import ARecord, AAAARecord, PTRRecord, CNameRecord, NSRecord, MXRecord, SRVRecord, TXTRecord
 from samba import sd_utils, descriptor
 
+
 class DnsserverTests(RpcInterfaceTestCase):
 
     @classmethod
     def setUpClass(cls):
         good_dns = ["SAMDOM.EXAMPLE.COM",
                     "1.EXAMPLE.COM",
-                    "%sEXAMPLE.COM" % ("1."*100),
+                    "%sEXAMPLE.COM" % ("1." * 100),
                     "EXAMPLE",
                     "\n.COM",
                     "!@#$%^&*()_",
@@ -102,7 +104,7 @@ class DnsserverTests(RpcInterfaceTestCase):
         # actually create these records.
         invalid_mx = ["SAMDOM.EXAMPLE.COM -1",
                       "SAMDOM.EXAMPLE.COM 65536",
-                      "%s 1" % "A"*256]
+                      "%s 1" % ("A" * 256)]
         invalid_srv = ["SAMDOM.EXAMPLE.COM 0 65536 0",
                        "SAMDOM.EXAMPLE.COM 0 0 65536",
                        "SAMDOM.EXAMPLE.COM 65536 0 0"]
@@ -120,11 +122,10 @@ class DnsserverTests(RpcInterfaceTestCase):
                                         self.get_credentials())
 
         self.samdb = SamDB(url="ldap://%s" % os.environ["DC_SERVER_IP"],
-                           lp = self.get_loadparm(),
+                           lp=self.get_loadparm(),
                            session_info=system_session(),
                            credentials=self.get_credentials())
 
-
         self.custom_zone = "zone"
         zone_create_info = dnsserver.DNS_RPC_ZONE_CREATE_INFO_LONGHORN()
         zone_create_info.pszZoneName = self.custom_zone
@@ -171,7 +172,7 @@ class DnsserverTests(RpcInterfaceTestCase):
         self.add_record(self.custom_zone, "testrecord", record_type_str, record_str)
 
         dn, record = self.get_record_from_db(self.custom_zone, "testrecord")
-        record.rank = 0 # DNS_RANK_NONE
+        record.rank = 0  # DNS_RANK_NONE
         res = self.samdb.dns_replace_by_dn(dn, [record])
         if res is not None:
             self.fail("Unable to update dns record to have DNS_RANK_NONE.")
@@ -211,7 +212,7 @@ class DnsserverTests(RpcInterfaceTestCase):
 
         zone_dn = None
         for zone in zones:
-            if zone_name in str(zone.dn):
+            if "DC=%s," % zone_name in str(zone.dn):
                 zone_dn = zone.dn
                 break
 
@@ -224,7 +225,8 @@ class DnsserverTests(RpcInterfaceTestCase):
 
         for old_packed_record in records:
             if record_name in str(old_packed_record.dn):
-                return (old_packed_record.dn, ndr_unpack(dnsp.DnssrvRpcRecord, old_packed_record["dnsRecord"][0]))
+                rec = ndr_unpack(dnsp.DnssrvRpcRecord, old_packed_record["dnsRecord"][0])
+                return (old_packed_record.dn, rec)
 
     def test_duplicate_matching(self):
         """
@@ -322,6 +324,35 @@ class DnsserverTests(RpcInterfaceTestCase):
                 self.assert_num_records(self.custom_zone, "testrecord", record_type_str)
                 self.delete_record(self.custom_zone, "testrecord", record_type_str, record_str)
 
+    def check_params(self, wDataLength, rank, flags, dwTtlSeconds, dwReserved, data,
+                     wType, dwTimeStamp=0, zone="zone", rec_name="testrecord"):
+        res = self.get_record_from_db(zone, rec_name)
+        self.assertIsNotNone(res, "Expected record %s but was not found over LDAP." % data)
+        (rec_dn, rec) = res
+        self.assertEqual(wDataLength, rec.wDataLength, "Unexpected data length for record %s. Got %s, expected %s." % (data, rec.wDataLength, wDataLength))
+        self.assertEqual(rank, rec.rank, "Unexpected rank for record %s. Got %s, expected %s." % (data, rec.rank, rank))
+        self.assertEqual(flags, rec.flags, "Unexpected flags for record %s. Got %s, expected %s." % (data, rec.flags, flags))
+        self.assertEqual(dwTtlSeconds, rec.dwTtlSeconds, "Unexpected time to live for record %s. Got %s, expected %s." % (data, rec.dwTtlSeconds, dwTtlSeconds))
+        self.assertEqual(dwReserved, rec.dwReserved, "Unexpected dwReserved for record %s. Got %s, expected %s." % (data, rec.dwReserved, dwReserved))
+        self.assertEqual(data.lower(), rec.data.lower(), "Unexpected data for record %s. Got %s, expected %s." % (data, rec.data.lower(), data.lower()))
+        self.assertEqual(wType, rec.wType, "Unexpected wType for record %s. Got %s, expected %s." % (data, rec.wType, wType))
+        self.assertEqual(dwTimeStamp, rec.dwTimeStamp, "Unexpected timestamp for record %s. Got %s, expected %s." % (data, rec.dwTimeStamp, dwTimeStamp))
+
+    def test_record_params(self):
+        """
+        Make sure that, when we add records to the database,
+        they're added with reasonable parameters.
+        """
+        self.add_record(self.custom_zone, "testrecord", "A", "192.168.50.50")
+        self.check_params(4, 240, 0, 900, 0, "192.168.50.50", 1)
+        self.delete_record(self.custom_zone, "testrecord", "A", "192.168.50.50")
+        self.add_record(self.custom_zone, "testrecord", "AAAA", "AAAA:AAAA::")
+        self.check_params(16, 240, 0, 900, 0, "AAAA:AAAA:0000:0000:0000:0000:0000:0000", 28)
+        self.delete_record(self.custom_zone, "testrecord", "AAAA", "AAAA:AAAA::")
+        self.add_record(self.custom_zone, "testrecord", "CNAME", "cnamedest")
+        self.check_params(13, 240, 0, 900, 0, "cnamedest", 5)
+        self.delete_record(self.custom_zone, "testrecord", "CNAME", "cnamedest")
+
     def test_reject_invalid_commands(self):
         """
         Make sure that we can't add a variety of invalid records,
@@ -337,7 +368,7 @@ class DnsserverTests(RpcInterfaceTestCase):
                     self.assert_num_records(self.custom_zone, "testrecord", record_type_str, expected_num=0)
                     self.delete_record(self.custom_zone, "testrecord", record_type_str, record_str, assertion=False)
                 except AssertionError as e:
-                    print e
+                    print(e)
                     num_failures = num_failures + 1
 
         # Also try to update valid records to invalid ones, making sure this fails
@@ -348,7 +379,7 @@ class DnsserverTests(RpcInterfaceTestCase):
                 try:
                     self.add_record(self.custom_zone, "testrecord", record_type_str, record_str, assertion=False)
                 except AssertionError as e:
-                    print e
+                    print(e)
                     num_failures = num_failures + 1
                 self.delete_record(self.custom_zone, "testrecord", record_type_str, good_record_str)
 
@@ -485,7 +516,7 @@ class DnsserverTests(RpcInterfaceTestCase):
         """
         for record_type_str in self.good_records:
             for i in range(1, len(self.good_records[record_type_str])):
-                record1 = self.good_records[record_type_str][i-1]
+                record1 = self.good_records[record_type_str][i - 1]
                 record2 = self.good_records[record_type_str][i]
 
                 if record_type_str == 'CNAME':
@@ -642,12 +673,12 @@ class DnsserverTests(RpcInterfaceTestCase):
 
         try:
             self.conn.DnssrvUpdateRecord2(client_version,
-                                                   0,
-                                                   self.server,
-                                                   zone,
-                                                   name,
-                                                   None,
-                                                   del_rec_buf)
+                                          0,
+                                          self.server,
+                                          zone,
+                                          name,
+                                          None,
+                                          del_rec_buf)
             if not assertion:
                 raise AssertionError("Successfully deleted record '%s' of type '%s', which should have failed." % (record_str, record_type_str))
         except RuntimeError as e:
@@ -689,16 +720,16 @@ class DnsserverTests(RpcInterfaceTestCase):
 
         # Create zone
         self.conn.DnssrvOperation2(client_version,
-                                    0,
-                                    self.server,
-                                    None,
-                                    0,
-                                    'ZoneCreate',
-                                    dnsserver.DNSSRV_TYPEID_ZONE_CREATE,
-                                    zone_create)
+                                   0,
+                                   self.server,
+                                   None,
+                                   0,
+                                   'ZoneCreate',
+                                   dnsserver.DNSSRV_TYPEID_ZONE_CREATE,
+                                   zone_create)
 
         request_filter = (dnsserver.DNS_ZONE_REQUEST_REVERSE |
-                            dnsserver.DNS_ZONE_REQUEST_PRIMARY)
+                          dnsserver.DNS_ZONE_REQUEST_PRIMARY)
         _, zones = self.conn.DnssrvComplexOperation2(client_version,
                                                      0,
                                                      self.server,
@@ -710,48 +741,47 @@ class DnsserverTests(RpcInterfaceTestCase):
 
         # Delete zone
         self.conn.DnssrvOperation2(client_version,
-                                    0,
-                                    self.server,
-                                    rev_zone,
-                                    0,
-                                    'DeleteZoneFromDs',
-                                    dnsserver.DNSSRV_TYPEID_NULL,
-                                    None)
+                                   0,
+                                   self.server,
+                                   rev_zone,
+                                   0,
+                                   'DeleteZoneFromDs',
+                                   dnsserver.DNSSRV_TYPEID_NULL,
+                                   None)
 
         typeid, zones = self.conn.DnssrvComplexOperation2(client_version,
-                                                            0,
-                                                            self.server,
-                                                            None,
-                                                            'EnumZones',
-                                                            dnsserver.DNSSRV_TYPEID_DWORD,
-                                                            request_filter)
+                                                          0,
+                                                          self.server,
+                                                          None,
+                                                          'EnumZones',
+                                                          dnsserver.DNSSRV_TYPEID_DWORD,
+                                                          request_filter)
         self.assertEquals(0, zones.dwZoneCount)
 
-
     def test_complexoperation2(self):
         client_version = dnsserver.DNS_CLIENT_VERSION_LONGHORN
         request_filter = (dnsserver.DNS_ZONE_REQUEST_FORWARD |
-                            dnsserver.DNS_ZONE_REQUEST_PRIMARY)
+                          dnsserver.DNS_ZONE_REQUEST_PRIMARY)
 
         typeid, zones = self.conn.DnssrvComplexOperation2(client_version,
-                                                            0,
-                                                            self.server,
-                                                            None,
-                                                            'EnumZones',
-                                                            dnsserver.DNSSRV_TYPEID_DWORD,
-                                                            request_filter)
+                                                          0,
+                                                          self.server,
+                                                          None,
+                                                          'EnumZones',
+                                                          dnsserver.DNSSRV_TYPEID_DWORD,
+                                                          request_filter)
         self.assertEquals(dnsserver.DNSSRV_TYPEID_ZONE_LIST, typeid)
         self.assertEquals(3, zones.dwZoneCount)
 
         request_filter = (dnsserver.DNS_ZONE_REQUEST_REVERSE |
-                            dnsserver.DNS_ZONE_REQUEST_PRIMARY)
+                          dnsserver.DNS_ZONE_REQUEST_PRIMARY)
         typeid, zones = self.conn.DnssrvComplexOperation2(client_version,
-                                                            0,
-                                                            self.server,
-                                                            None,
-                                                            'EnumZones',
-                                                            dnsserver.DNSSRV_TYPEID_DWORD,
-                                                            request_filter)
+                                                          0,
+                                                          self.server,
+                                                          None,
+                                                          'EnumZones',
+                                                          dnsserver.DNSSRV_TYPEID_DWORD,
+                                                          request_filter)
         self.assertEquals(dnsserver.DNSSRV_TYPEID_ZONE_LIST, typeid)
         self.assertEquals(0, zones.dwZoneCount)
 
@@ -785,12 +815,12 @@ class DnsserverTests(RpcInterfaceTestCase):
         add_rec_buf = dnsserver.DNS_RPC_RECORD_BUF()
         add_rec_buf.rec = rec
         self.conn.DnssrvUpdateRecord2(client_version,
-                                        0,
-                                        self.server,
-                                        self.zone,
-                                        name,
-                                        add_rec_buf,
-                                        None)
+                                      0,
+                                      self.server,
+                                      self.zone,
+                                      name,
+                                      add_rec_buf,
+                                      None)
 
         _, result = self.conn.DnssrvEnumRecords2(client_version,
                                                  0,
@@ -813,23 +843,23 @@ class DnsserverTests(RpcInterfaceTestCase):
         del_rec_buf = dnsserver.DNS_RPC_RECORD_BUF()
         del_rec_buf.rec = rec
         self.conn.DnssrvUpdateRecord2(client_version,
-                                        0,
-                                        self.server,
-                                        self.zone,
-                                        name,
-                                        add_rec_buf,
-                                        del_rec_buf)
+                                      0,
+                                      self.server,
+                                      self.zone,
+                                      name,
+                                      add_rec_buf,
+                                      del_rec_buf)
 
         buflen, result = self.conn.DnssrvEnumRecords2(client_version,
-                                                        0,
-                                                        self.server,
-                                                        self.zone,
-                                                        name,
-                                                        None,
-                                                        record_type,
-                                                        select_flags,
-                                                        None,
-                                                        None)
+                                                      0,
+                                                      self.server,
+                                                      self.zone,
+                                                      name,
+                                                      None,
+                                                      record_type,
+                                                      select_flags,
+                                                      None,
+                                                      None)
         self.assertEquals(1, result.count)
         self.assertEquals(1, result.rec[0].wRecordCount)
         self.assertEquals(dnsp.DNS_TYPE_A, result.rec[0].records[0].wType)
@@ -839,24 +869,24 @@ class DnsserverTests(RpcInterfaceTestCase):
         del_rec_buf = dnsserver.DNS_RPC_RECORD_BUF()
         del_rec_buf.rec = rec2
         self.conn.DnssrvUpdateRecord2(client_version,
-                                        0,
-                                        self.server,
-                                        self.zone,
-                                        name,
-                                        None,
-                                        del_rec_buf)
+                                      0,
+                                      self.server,
+                                      self.zone,
+                                      name,
+                                      None,
+                                      del_rec_buf)
 
         self.assertRaises(RuntimeError, self.conn.DnssrvEnumRecords2,
-                                        client_version,
-                                        0,
-                                        self.server,
-                                        self.zone,
-                                        name,
-                                        None,
-                                        record_type,
-                                        select_flags,
-                                        None,
-                                        None)
+                          client_version,
+                          0,
+                          self.server,
+                          self.zone,
+                          name,
+                          None,
+                          record_type,
+                          select_flags,
+                          None,
+                          None)
 
     # The following tests do not pass against Samba because the owner and
     # group are not consistent with Windows, as well as some ACEs.
@@ -891,11 +921,11 @@ class DnsserverTests(RpcInterfaceTestCase):
 
         dns_admin = str(ndr_unpack(security.dom_sid, res[0]['objectSid'][0]))
 
-        packed_sd = descriptor.sddl2binary("O:SYG:BA" \
-                                           "D:AI(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;DA)" \
-                                           "(A;;CC;;;AU)" \
-                                           "(A;;RPLCLORC;;;WD)" \
-                                           "(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;SY)" \
+        packed_sd = descriptor.sddl2binary("O:SYG:BA"
+                                           "D:AI(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;DA)"
+                                           "(A;;CC;;;AU)"
+                                           "(A;;RPLCLORC;;;WD)"
+                                           "(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;SY)"
                                            "(A;CI;RPWPCRCCDCLCRCWOWDSDDTSW;;;ED)",
                                            domain_sid, {"DnsAdmins": dns_admin})
         expected_sd = descriptor.get_clean_sd(ndr_unpack(security.descriptor, packed_sd))
@@ -951,11 +981,11 @@ class DnsserverTests(RpcInterfaceTestCase):
 
         dns_admin = str(ndr_unpack(security.dom_sid, res[0]['objectSid'][0]))
 
-        packed_sd = descriptor.sddl2binary("O:DAG:DA" \
-                                           "D:AI(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;DA)" \
-                                           "(A;;CC;;;AU)" \
-                                           "(A;;RPLCLORC;;;WD)" \
-                                           "(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;SY)" \
+        packed_sd = descriptor.sddl2binary("O:DAG:DA"
+                                           "D:AI(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;DA)"
+                                           "(A;;CC;;;AU)"
+                                           "(A;;RPLCLORC;;;WD)"
+                                           "(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;SY)"
                                            "(A;CI;RPWPCRCCDCLCRCWOWDSDDTSW;;;ED)",
                                            domain_sid, {"DnsAdmins": dns_admin})
         expected_sd = descriptor.get_clean_sd(ndr_unpack(security.descriptor, packed_sd))
@@ -969,7 +999,7 @@ class DnsserverTests(RpcInterfaceTestCase):
                                                               packed_part_sd))
         try:
             msdns_dn = ldb.Dn(self.samdb, "CN=MicrosoftDNS,%s" % str(partition_dn))
-            security_desc_dict = [(current_dn.get_linearized(),  expected_sd),
+            security_desc_dict = [(current_dn.get_linearized(), expected_sd),
                                   (msdns_dn.get_linearized(), expected_msdns_sd),
                                   (partition_dn.get_linearized(), expected_part_sd)]
 
@@ -1023,11 +1053,11 @@ class DnsserverTests(RpcInterfaceTestCase):
 
         dns_admin = str(ndr_unpack(security.dom_sid, res[0]['objectSid'][0]))
 
-        packed_sd = descriptor.sddl2binary("O:DAG:DA" \
-                                           "D:AI(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;DA)" \
-                                           "(A;;CC;;;AU)" \
-                                           "(A;;RPLCLORC;;;WD)" \
-                                           "(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;SY)" \
+        packed_sd = descriptor.sddl2binary("O:DAG:DA"
+                                           "D:AI(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;DA)"
+                                           "(A;;CC;;;AU)"
+                                           "(A;;RPLCLORC;;;WD)"
+                                           "(A;;RPWPCRCCDCLCLORCWOWDSDDTSW;;;SY)"
                                            "(A;CI;RPWPCRCCDCLCRCWOWDSDDTSW;;;ED)",
                                            domain_sid, {"DnsAdmins": dns_admin})
         expected_sd = descriptor.get_clean_sd(ndr_unpack(security.descriptor, packed_sd))
@@ -1041,7 +1071,7 @@ class DnsserverTests(RpcInterfaceTestCase):
                                                               packed_part_sd))
 
         msdns_dn = ldb.Dn(self.samdb, "CN=MicrosoftDNS,%s" % str(partition_dn))
-        security_desc_dict = [(current_dn.get_linearized(),  expected_sd),
+        security_desc_dict = [(current_dn.get_linearized(), expected_sd),
                               (msdns_dn.get_linearized(), expected_msdns_sd),
                               (partition_dn.get_linearized(), expected_part_sd)]
 
