@@ -825,6 +825,7 @@ SMBC_opendir_ctx(SMBCCTX *context,
 				}
 			} else if (srv ||
                                    (resolve_name(server, &rem_ss, 0x20, false))) {
+				int rc;
 
                                 /*
                                  * If we hadn't found the server, get one now
@@ -851,24 +852,24 @@ SMBC_opendir_ctx(SMBCCTX *context,
 
                                 /* List the shares ... */
 
-                                if (net_share_enum_rpc(
-                                            srv->cli,
-                                            list_fn,
-                                            (void *) dir) < 0 &&
-                                    cli_RNetShareEnum(
-                                            srv->cli,
-                                            list_fn,
-                                            (void *)dir) < 0) {
-
-                                        errno = cli_errno(srv->cli);
-                                        if (dir) {
-                                                SAFE_FREE(dir->fname);
-                                                SAFE_FREE(dir);
-                                        }
+				rc = net_share_enum_rpc(srv->cli,
+							list_fn,
+							(void *)dir);
+				if (rc != 0 &&
+				    lp_client_min_protocol() <= PROTOCOL_NT1) {
+					rc = cli_RNetShareEnum(srv->cli,
+							       list_fn,
+							       (void *)dir);
+				}
+				if (rc != 0) {
+					errno = cli_errno(srv->cli);
+					if (dir != NULL) {
+						SAFE_FREE(dir->fname);
+						SAFE_FREE(dir);
+					}
 					TALLOC_FREE(frame);
-                                        return NULL;
-
-                                }
+					return NULL;
+				}
                         } else {
                                 /* Neither the workgroup nor server exists */
                                 errno = ECONNREFUSED;
@@ -1173,17 +1174,6 @@ SMBC_readdir_ctx(SMBCCTX *context,
 
         dir->dir_next = dir->dir_next->next;
 
-	/*
-	 * If we are returning file entries, we
-	 * have a duplicate list in dirplus.
-	 *
-	 * Update dirplus_next also so readdir and
-	 * readdirplus are kept in sync.
-	 */
-	if (dir->dirplus_list != NULL) {
-		dir->dirplus_next = dir->dirplus_next->next;
-	}
-
 	TALLOC_FREE(frame);
         return dirp;
 }
@@ -1230,17 +1220,6 @@ SMBC_readdirplus_ctx(SMBCCTX *context,
 	}
 	dir->dirplus_next = dir->dirplus_next->next;
 
-	/*
-	 * If we are returning file entries, we
-	 * have a duplicate list in dir_list
-	 *
-	 * Update dir_next also so readdir and
-	 * readdirplus are kept in sync.
-	 */
-	if (dir->dir_list) {
-		dir->dir_next = dir->dir_next->next;
-	}
-
 	TALLOC_FREE(frame);
 	return smb_finfo;
 }
@@ -1357,17 +1336,6 @@ SMBC_getdents_ctx(SMBCCTX *context,
 		}
 
 		dir->dir_next = dirlist = dirlist -> next;
-
-		/*
-		 * If we are returning file entries, we
-		 * have a duplicate list in dirplus.
-		 *
-		 * Update dirplus_next also so readdir and
-		 * readdirplus are kept in sync.
-		 */
-		if (dir->dirplus_list != NULL) {
-			dir->dirplus_next = dir->dirplus_next->next;
-		}
 	}
 
 	TALLOC_FREE(frame);
@@ -1671,43 +1639,35 @@ SMBC_telldir_ctx(SMBCCTX *context,
 
 /*
  * A routine to run down the list and see if the entry is OK
- * Modifies the dir list and the dirplus list (if it exists)
- * to point at the correct next entry on success.
  */
 
-static bool update_dir_ents(SMBCFILE *dir, struct smbc_dirent *dirent)
+static struct smbc_dir_list *
+check_dir_ent(struct smbc_dir_list *list,
+              struct smbc_dirent *dirent)
 {
-	struct smbc_dir_list *tmp_dir = dir->dir_list;
-	struct smbc_dirplus_list *tmp_dirplus = dir->dirplus_list;
 
-	/*
-	 * Run down the list looking for what we want.
-	 * If we're enumerating files both dir_list
-	 * and dirplus_list contain the same entry
-	 * list, as they were seeded from the same
-	 * cli_list callback.
-	 *
-	 * If we're enumerating servers then
-	 * dirplus_list will be NULL, so don't
-	 * update in that case.
-	 */
+	/* Run down the list looking for what we want */
+
+	if (dirent) {
+
+		struct smbc_dir_list *tmp = list;
+
+		while (tmp) {
+
+			if (tmp->dirent == dirent)
+				return tmp;
+
+			tmp = tmp->next;
 
-	while (tmp_dir != NULL) {
-		if (tmp_dir->dirent == dirent) {
-			dir->dir_next = tmp_dir;
-			if (tmp_dirplus != NULL) {
-				dir->dirplus_next = tmp_dirplus;
-			}
-			return true;
-		}
-		tmp_dir = tmp_dir->next;
-		if (tmp_dirplus != NULL) {
-			tmp_dirplus = tmp_dirplus->next;
 		}
+
 	}
-	return false;
+
+	return NULL;  /* Not found, or an error */
+
 }
 
+
 /*
  * Routine to seek on a directory
  */
@@ -1719,8 +1679,8 @@ SMBC_lseekdir_ctx(SMBCCTX *context,
 {
 	long int l_offset = offset;  /* Handle problems of size */
 	struct smbc_dirent *dirent = (struct smbc_dirent *)l_offset;
+	struct smbc_dir_list *list_ent = (struct smbc_dir_list *)NULL;
 	TALLOC_CTX *frame = talloc_stackframe();
-	bool ok;
 
 	if (!context || !context->internal->initialized) {
 
@@ -1743,10 +1703,6 @@ SMBC_lseekdir_ctx(SMBCCTX *context,
 	if (dirent == NULL) {  /* Seek to the begining of the list */
 
 		dir->dir_next = dir->dir_list;
-
-		/* Do the same for dirplus. */
-		dir->dirplus_next = dir->dirplus_list;
-
 		TALLOC_FREE(frame);
 		return 0;
 
@@ -1754,26 +1710,21 @@ SMBC_lseekdir_ctx(SMBCCTX *context,
 
         if (offset == -1) {     /* Seek to the end of the list */
                 dir->dir_next = NULL;
-
-		/* Do the same for dirplus. */
-		dir->dirplus_next = NULL;
-
 		TALLOC_FREE(frame);
                 return 0;
         }
 
-        /*
-         * Run down the list and make sure that the entry is OK.
-         * Update the position of both dir and dirplus lists.
-         */
+	/* Now, run down the list and make sure that the entry is OK       */
+	/* This may need to be changed if we change the format of the list */
 
-	ok = update_dir_ents(dir, dirent);
-	if (!ok) {
+	if ((list_ent = check_dir_ent(dir->dir_list, dirent)) == NULL) {
 		errno = EINVAL;   /* Bad entry */
 		TALLOC_FREE(frame);
 		return -1;
 	}
 
+	dir->dir_next = list_ent;
+
 	TALLOC_FREE(frame);
 	return 0;
 }
