@@ -673,71 +673,6 @@ static NTSTATUS gpfsacl_get_nt_acl(vfs_handle_struct *handle,
 	return map_nt_error_from_unix(errno);
 }
 
-static bool vfs_gpfs_nfs4_ace_to_gpfs_ace(SMB_ACE4PROP_T *nfs4_ace,
-					  struct gpfs_ace_v4 *gace,
-					  uid_t owner_uid)
-{
-	gace->aceType = nfs4_ace->aceType;
-	gace->aceFlags = nfs4_ace->aceFlags;
-	gace->aceMask = nfs4_ace->aceMask;
-
-	if (nfs4_ace->flags & SMB_ACE4_ID_SPECIAL) {
-		switch(nfs4_ace->who.special_id) {
-		case SMB_ACE4_WHO_EVERYONE:
-			gace->aceIFlags = ACE4_IFLAG_SPECIAL_ID;
-			gace->aceWho = ACE4_SPECIAL_EVERYONE;
-			break;
-		case SMB_ACE4_WHO_OWNER:
-			/*
-			 * With GPFS it is not possible to deny ACL or
-			 * attribute access to the owner. Setting an
-			 * ACL with such an entry is not possible.
-			 * Denying ACL or attribute access for the
-			 * owner through a named ACL entry can be
-			 * stored in an ACL, it is just not effective.
-			 *
-			 * Map this case to a named entry to allow at
-			 * least setting this ACL, which will be
-			 * enforced by the smbd permission check. Do
-			 * not do this for an inheriting OWNER entry,
-			 * as this represents a CREATOR OWNER ACE. The
-			 * remaining limitation is that CREATOR OWNER
-			 * cannot deny ACL or attribute access.
-			 */
-			if (!nfs_ace_is_inherit(nfs4_ace) &&
-			    nfs4_ace->aceType ==
-					SMB_ACE4_ACCESS_DENIED_ACE_TYPE &&
-			    nfs4_ace->aceMask & (SMB_ACE4_READ_ATTRIBUTES|
-						 SMB_ACE4_WRITE_ATTRIBUTES|
-						 SMB_ACE4_READ_ACL|
-						 SMB_ACE4_WRITE_ACL)) {
-				gace->aceIFlags = 0;
-				gace->aceWho = owner_uid;
-			} else {
-				gace->aceIFlags = ACE4_IFLAG_SPECIAL_ID;
-				gace->aceWho = ACE4_SPECIAL_OWNER;
-			}
-			break;
-		case SMB_ACE4_WHO_GROUP:
-			gace->aceIFlags = ACE4_IFLAG_SPECIAL_ID;
-			gace->aceWho = ACE4_SPECIAL_GROUP;
-			break;
-		default:
-			DBG_WARNING("Unsupported special_id %d\n",
-				    nfs4_ace->who.special_id);
-			return false;
-		}
-
-		return true;
-	}
-
-	gace->aceIFlags = 0;
-	gace->aceWho = (nfs4_ace->aceFlags & SMB_ACE4_IDENTIFIER_GROUP) ?
-		nfs4_ace->who.gid : nfs4_ace->who.uid;
-
-	return true;
-}
-
 static struct gpfs_acl *vfs_gpfs_smbacl2gpfsacl(TALLOC_CTX *mem_ctx,
 						files_struct *fsp,
 						struct SMB4ACL_T *smbacl,
@@ -770,12 +705,58 @@ static struct gpfs_acl *vfs_gpfs_smbacl2gpfsacl(TALLOC_CTX *mem_ctx,
 	for (smbace=smb_first_ace4(smbacl); smbace!=NULL; smbace = smb_next_ace4(smbace)) {
 		struct gpfs_ace_v4 *gace = gpfs_ace_ptr(gacl, gacl->acl_nace);
 		SMB_ACE4PROP_T	*aceprop = smb_get_ace4(smbace);
-		bool add_ace;
 
-		add_ace = vfs_gpfs_nfs4_ace_to_gpfs_ace(aceprop, gace,
-							fsp->fsp_name->st.st_ex_uid);
-		if (!add_ace) {
-			continue;
+		gace->aceType = aceprop->aceType;
+		gace->aceFlags = aceprop->aceFlags;
+		gace->aceMask = aceprop->aceMask;
+
+		/*
+		 * GPFS can't distinguish between WRITE and APPEND on
+		 * files, so one being set without the other is an
+		 * error. Sorry for the many ()'s :-)
+		 */
+
+		if (!fsp->is_directory
+		    &&
+		    ((((gace->aceMask & ACE4_MASK_WRITE) == 0)
+		      && ((gace->aceMask & ACE4_MASK_APPEND) != 0))
+		     ||
+		     (((gace->aceMask & ACE4_MASK_WRITE) != 0)
+		      && ((gace->aceMask & ACE4_MASK_APPEND) == 0)))
+		    &&
+		    lp_parm_bool(fsp->conn->params->service, "gpfs",
+				 "merge_writeappend", True)) {
+			DEBUG(2, ("vfs_gpfs.c: file [%s]: ACE contains "
+				  "WRITE^APPEND, setting WRITE|APPEND\n",
+				  fsp_str_dbg(fsp)));
+			gace->aceMask |= ACE4_MASK_WRITE|ACE4_MASK_APPEND;
+		}
+
+		gace->aceIFlags = (aceprop->flags&SMB_ACE4_ID_SPECIAL) ? ACE4_IFLAG_SPECIAL_ID : 0;
+
+		if (aceprop->flags&SMB_ACE4_ID_SPECIAL)
+		{
+			switch(aceprop->who.special_id)
+			{
+			case SMB_ACE4_WHO_EVERYONE:
+				gace->aceWho = ACE4_SPECIAL_EVERYONE;
+				break;
+			case SMB_ACE4_WHO_OWNER:
+				gace->aceWho = ACE4_SPECIAL_OWNER;
+				break;
+			case SMB_ACE4_WHO_GROUP:
+				gace->aceWho = ACE4_SPECIAL_GROUP;
+				break;
+			default:
+				DEBUG(8, ("unsupported special_id %d\n", aceprop->who.special_id));
+				continue; /* don't add it !!! */
+			}
+		} else {
+			/* just only for the type safety... */
+			if (aceprop->aceFlags&SMB_ACE4_IDENTIFIER_GROUP)
+				gace->aceWho = aceprop->who.gid;
+			else
+				gace->aceWho = aceprop->who.uid;
 		}
 
 		gacl->acl_nace++;
@@ -2097,6 +2078,7 @@ static int vfs_gpfs_connect(struct vfs_handle_struct *handle,
 {
 	struct gpfs_config_data *config;
 	int ret;
+	bool check_fstype;
 
 	gpfswrap_lib_init(0);
 
@@ -2113,6 +2095,31 @@ static int vfs_gpfs_connect(struct vfs_handle_struct *handle,
 		return ret;
 	}
 
+	check_fstype = lp_parm_bool(SNUM(handle->conn), "gpfs",
+				    "check_fstype", true);
+
+	if (check_fstype && !IS_IPC(handle->conn)) {
+		const char *connectpath = handle->conn->connectpath;
+		struct statfs buf = { 0 };
+
+		ret = statfs(connectpath, &buf);
+		if (ret != 0) {
+			DBG_ERR("statfs failed for share %s at path %s: %s\n",
+				service, connectpath, strerror(errno));
+			TALLOC_FREE(config);
+			return ret;
+		}
+
+		if (buf.f_type != GPFS_SUPER_MAGIC) {
+			DBG_ERR("SMB share %s, path %s not in GPFS file system."
+				" statfs magic: 0x%lx\n",
+				service, connectpath, buf.f_type);
+			errno = EINVAL;
+			TALLOC_FREE(config);
+			return -1;
+		}
+	}
+
 	ret = smbacl4_get_vfs_params(handle->conn, &config->nfs4_params);
 	if (ret < 0) {
 		TALLOC_FREE(config);
@@ -2182,6 +2189,12 @@ static int vfs_gpfs_connect(struct vfs_handle_struct *handle,
 		}
 	}
 
+	/*
+	 * Unless we have an async implementation of get_dos_attributes turn
+	 * this off.
+	 */
+	lp_do_parameter(SNUM(handle->conn), "smbd:async dosmode", "false");
+
 	return 0;
 }
 
@@ -2580,6 +2593,8 @@ static struct vfs_fn_pointers vfs_gpfs_fns = {
 	.linux_setlease_fn = vfs_gpfs_setlease,
 	.get_real_filename_fn = vfs_gpfs_get_real_filename,
 	.get_dos_attributes_fn = vfs_gpfs_get_dos_attributes,
+	.get_dos_attributes_send_fn = vfs_not_implemented_get_dos_attributes_send,
+	.get_dos_attributes_recv_fn = vfs_not_implemented_get_dos_attributes_recv,
 	.fget_dos_attributes_fn = vfs_gpfs_fget_dos_attributes,
 	.set_dos_attributes_fn = vfs_gpfs_set_dos_attributes,
 	.fset_dos_attributes_fn = vfs_gpfs_fset_dos_attributes,
