@@ -204,7 +204,7 @@ bool print_backend_init(struct messaging_context *msg_ctx)
 		return false;
 	}
 
-	print_cache_path = cache_path("printing");
+	print_cache_path = cache_path(talloc_tos(), "printing");
 	if (print_cache_path == NULL) {
 		return false;
 	}
@@ -214,7 +214,7 @@ bool print_backend_init(struct messaging_context *msg_ctx)
 		return false;
 	}
 
-	print_cache_path = cache_path("printing.tdb");
+	print_cache_path = cache_path(talloc_tos(), "printing.tdb");
 	if (print_cache_path == NULL) {
 		return false;
 	}
@@ -352,7 +352,7 @@ static int unpack_devicemode(TALLOC_CTX *mem_ctx,
 	struct spoolss_DeviceMode *dm;
 	enum ndr_err_code ndr_err;
 	char *data = NULL;
-	int data_len = 0;
+	uint32_t data_len = 0;
 	DATA_BLOB blob;
 	int len = 0;
 
@@ -991,7 +991,7 @@ static void print_unix_job(struct tevent_context *ev,
 
 struct traverse_struct {
 	print_queue_struct *queue;
-	int qcount, snum, maxcount, total_jobs;
+	size_t qcount, snum, maxcount, total_jobs;
 	const char *sharename;
 	time_t lpq_time;
 	const char *lprm_command;
@@ -1010,7 +1010,7 @@ static int traverse_fn_delete(TDB_CONTEXT *t, TDB_DATA key, TDB_DATA data, void
 	struct traverse_struct *ts = (struct traverse_struct *)state;
 	struct printjob pjob;
 	uint32_t jobid;
-	int i = 0;
+	size_t i = 0;
 
 	if (  key.dsize != sizeof(jobid) )
 		return 0;
@@ -1408,7 +1408,7 @@ static void print_queue_update_internal(struct tevent_context *ev,
                                         struct printif *current_printif,
                                         char *lpq_command, char *lprm_command)
 {
-	int i, qcount;
+	size_t i, qcount;
 	print_queue_struct *queue = NULL;
 	print_status_struct status;
 	print_status_struct old_status;
@@ -1443,8 +1443,10 @@ static void print_queue_update_internal(struct tevent_context *ev,
 		current_printif->type,
 		lpq_command, &queue, &status);
 
-	DEBUG(3, ("print_queue_update_internal: %d job%s in queue for %s\n",
-		qcount, (qcount != 1) ?	"s" : "", sharename));
+	DBG_NOTICE("%zu job%s in queue for %s\n",
+		   qcount,
+		   (qcount != 1) ? "s" : "",
+		   sharename);
 
 	/* Sort the queue by submission time otherwise they are displayed
 	   in hash order. */
@@ -1519,15 +1521,20 @@ static void print_queue_update_internal(struct tevent_context *ev,
 	SAFE_FREE(tstruct.queue);
 	talloc_free(tmp_ctx);
 
-	DEBUG(10,("print_queue_update_internal: printer %s INFO/total_jobs = %d\n",
-				sharename, tstruct.total_jobs ));
+	DBG_DEBUG("printer %s INFO, total_jobs = %zu\n",
+		  sharename,
+		  tstruct.total_jobs);
 
 	tdb_store_int32(pdb->tdb, "INFO/total_jobs", tstruct.total_jobs);
 
 	get_queue_status(sharename, &old_status);
-	if (old_status.qcount != qcount)
-		DEBUG(10,("print_queue_update_internal: queue status change %d jobs -> %d jobs for printer %s\n",
-					old_status.qcount, qcount, sharename));
+	if (old_status.qcount != qcount) {
+		DBG_DEBUG("Queue status change %zu jobs -> %zu jobs "
+			  "for printer %s\n",
+			  old_status.qcount,
+			  qcount,
+			  sharename);
+	}
 
 	/* store the new queue status structure */
 	slprintf(keystr, sizeof(keystr)-1, "STATUS/%s", sharename);
@@ -1675,7 +1682,7 @@ void print_queue_receive(struct messaging_context *msg,
 		return;
 	}
 
-	print_queue_update_with_lock(server_event_context(), msg, sharename,
+	print_queue_update_with_lock(global_event_context(), msg, sharename,
 		get_printer_fns_from_type((enum printing_types)printing_type),
 		lpqcommand, lprmcommand );
 
@@ -1757,7 +1764,7 @@ static void print_queue_update(struct messaging_context *msg_ctx,
 	if ( force || background_lpq_updater_pid == -1 ) {
 		DEBUG(4,("print_queue_update: updating queue [%s] myself\n", sharename));
 		current_printif = get_printer_fns( snum );
-		print_queue_update_with_lock(server_event_context(), msg_ctx,
+		print_queue_update_with_lock(global_event_context(), msg_ctx,
 					     sharename, current_printif,
 					     lpqcommand, lprmcommand);
 
@@ -2300,7 +2307,7 @@ WERROR print_job_delete(const struct auth_session_info *server_info,
 		}
 	}
 
-	if (!print_job_delete1(server_event_context(), msg_ctx, snum, jobid)) {
+	if (!print_job_delete1(global_event_context(), msg_ctx, snum, jobid)) {
 		werr = WERR_ACCESS_DENIED;
 		goto err_out;
 	}
@@ -2380,7 +2387,7 @@ WERROR print_job_pause(const struct auth_session_info *server_info,
 
 	/* Send a printer notify message */
 
-	notify_job_status(server_event_context(), msg_ctx, sharename, jobid,
+	notify_job_status(global_event_context(), msg_ctx, sharename, jobid,
 			  JOB_STATUS_PAUSED);
 
 	/* how do we tell if this succeeded? */
@@ -2446,7 +2453,7 @@ WERROR print_job_resume(const struct auth_session_info *server_info,
 
 	/* Send a printer notify message */
 
-	notify_job_status(server_event_context(), msg_ctx, sharename, jobid,
+	notify_job_status(global_event_context(), msg_ctx, sharename, jobid,
 			  JOB_STATUS_QUEUED);
 
 	werr = WERR_OK;
@@ -2874,7 +2881,7 @@ WERROR print_job_start(const struct auth_session_info *server_info,
 		goto fail;
 	}
 
-	pjob_store(server_event_context(), msg_ctx, sharename, jobid, &pjob);
+	pjob_store(global_event_context(), msg_ctx, sharename, jobid, &pjob);
 
 	/* Update the 'jobs added' entry used by print_queue_status. */
 	add_to_jobs_added(pdb, jobid);
@@ -2889,7 +2896,7 @@ WERROR print_job_start(const struct auth_session_info *server_info,
 
 fail:
 	if (jobid != -1) {
-		pjob_delete(server_event_context(), msg_ctx, sharename, jobid);
+		pjob_delete(global_event_context(), msg_ctx, sharename, jobid);
 	}
 
 	release_print_db(pdb);
@@ -2923,7 +2930,7 @@ void print_job_endpage(struct messaging_context *msg_ctx,
 	}
 
 	pjob->page_count++;
-	pjob_store(server_event_context(), msg_ctx, sharename, jobid, pjob);
+	pjob_store(global_event_context(), msg_ctx, sharename, jobid, pjob);
 err_out:
 	talloc_free(tmp_ctx);
 }
@@ -3007,7 +3014,7 @@ NTSTATUS print_job_end(struct messaging_context *msg_ctx, int snum,
 		DEBUG(5,("print_job_end: canceling spool of %s (%s)\n",
 			pjob->filename, pjob->size ? "deleted" : "zero length" ));
 		unlink(pjob->filename);
-		pjob_delete(server_event_context(), msg_ctx, sharename, jobid);
+		pjob_delete(global_event_context(), msg_ctx, sharename, jobid);
 		return NT_STATUS_OK;
 	}
 
@@ -3045,7 +3052,7 @@ NTSTATUS print_job_end(struct messaging_context *msg_ctx, int snum,
 
 	pjob->spooled = True;
 	pjob->status = LPQ_QUEUED;
-	pjob_store(server_event_context(), msg_ctx, sharename, jobid, pjob);
+	pjob_store(global_event_context(), msg_ctx, sharename, jobid, pjob);
 
 	/* make sure the database is up to date */
 	if (print_cache_expired(lp_const_servicename(snum), True))
@@ -3059,7 +3066,7 @@ fail:
 	/* Still need to add proper error return propagation! 010122:JRR */
 	pjob->fd = -1;
 	unlink(pjob->filename);
-	pjob_delete(server_event_context(), msg_ctx, sharename, jobid);
+	pjob_delete(global_event_context(), msg_ctx, sharename, jobid);
 err_out:
 	talloc_free(tmp_ctx);
 	return status;
@@ -3337,7 +3344,7 @@ WERROR print_queue_pause(const struct auth_session_info *server_info,
 
 	/* Send a printer notify message */
 
-	notify_printer_status(server_event_context(), msg_ctx, snum,
+	notify_printer_status(global_event_context(), msg_ctx, snum,
 			      PRINTER_STATUS_PAUSED);
 
 	return WERR_OK;
@@ -3374,7 +3381,7 @@ WERROR print_queue_resume(const struct auth_session_info *server_info,
 
 	/* Send a printer notify message */
 
-	notify_printer_status(server_event_context(), msg_ctx, snum,
+	notify_printer_status(global_event_context(), msg_ctx, snum,
 			      PRINTER_STATUS_OK);
 
 	return WERR_OK;
@@ -3424,7 +3431,7 @@ WERROR print_queue_purge(const struct auth_session_info *server_info,
 				 jobid);
 
 		if (owner || can_job_admin) {
-			print_job_delete1(server_event_context(), msg_ctx,
+			print_job_delete1(global_event_context(), msg_ctx,
 					  snum, jobid);
 		}
 	}
