@@ -22,6 +22,7 @@
 #include "includes.h"
 #include "messages.h"
 #include "ntdomain.h"
+#include "passdb.h"
 
 #include "lib/id_cache.h"
 
@@ -242,7 +243,7 @@ static bool lsasd_child_init(struct tevent_context *ev_ctx,
 			     struct pf_worker_data *pf)
 {
 	NTSTATUS status;
-	struct messaging_context *msg_ctx = server_messaging_context();
+	struct messaging_context *msg_ctx = global_messaging_context();
 	bool ok;
 
 	status = reinit_after_fork(msg_ctx, ev_ctx,
@@ -251,6 +252,7 @@ static bool lsasd_child_init(struct tevent_context *ev_ctx,
 		DEBUG(0,("reinit_after_fork() failed\n"));
 		smb_panic("reinit_after_fork() failed");
 	}
+	initialize_password_db(true, ev_ctx);
 
 	lsasd_child_id = child_id;
 	lsasd_reopen_logs(child_id);
@@ -856,6 +858,7 @@ void start_lsasd(struct tevent_context *ev_ctx,
 		DEBUG(0,("reinit_after_fork() failed\n"));
 		smb_panic("reinit_after_fork() failed");
 	}
+	initialize_password_db(true, ev_ctx);
 
 	/* save the parent process id so the children can use it later */
 	parent_id = messaging_server_id(msg_ctx);
