@@ -284,18 +284,18 @@ static NTSTATUS get_domain_handle(struct rpc_pipe_client *cli,
 	struct dcerpc_binding_handle *b = cli->binding_handle;
 	NTSTATUS status = NT_STATUS_INVALID_PARAMETER, result;
 
-	if (StrCaseCmp(sam, "domain") == 0) {
+	if (strcasecmp_m(sam, "domain") == 0) {
 		status = dcerpc_samr_OpenDomain(b, mem_ctx,
 					      connect_pol,
 					      access_mask,
 					      _domain_sid,
 					      domain_pol,
 					      &result);
-	} else if (StrCaseCmp(sam, "builtin") == 0) {
+	} else if (strcasecmp_m(sam, "builtin") == 0) {
 		status = dcerpc_samr_OpenDomain(b, mem_ctx,
 					      connect_pol,
 					      access_mask,
-					      CONST_DISCARD(struct dom_sid2 *, &global_sid_Builtin),
+					      discard_const_p(struct dom_sid2, &global_sid_Builtin),
 					      domain_pol,
 					      &result);
 	}
@@ -316,10 +316,10 @@ static NTSTATUS cmd_samr_query_user(struct rpc_pipe_client *cli,
 {
 	struct policy_handle connect_pol, domain_pol, user_pol;
 	NTSTATUS status, result;
-	uint32 info_level = 21;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t info_level = 21;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	union samr_UserInfo *info = NULL;
-	uint32 user_rid = 0;
+	uint32_t user_rid = 0;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
 
 	if ((argc < 2) || (argc > 4)) {
@@ -539,9 +539,9 @@ static NTSTATUS cmd_samr_query_group(struct rpc_pipe_client *cli,
 	struct policy_handle connect_pol, domain_pol, group_pol;
 	NTSTATUS status, result;
 	enum samr_GroupInfoEnum info_level = GROUPINFOALL;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	union samr_GroupInfo *group_info = NULL;
-	uint32 group_rid;
+	uint32_t group_rid;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
 
 	if ((argc < 2) || (argc > 4)) {
@@ -624,8 +624,8 @@ static NTSTATUS cmd_samr_query_usergroups(struct rpc_pipe_client *cli,
 				domain_pol,
 				user_pol;
 	NTSTATUS status, result;
-	uint32 			user_rid;
-	uint32			access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t 		user_rid;
+	uint32_t		access_mask = MAXIMUM_ALLOWED_ACCESS;
 	int 			i;
 	struct samr_RidWithAttributeArray *rid_array = NULL;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
@@ -710,8 +710,8 @@ static NTSTATUS cmd_samr_query_useraliases(struct rpc_pipe_client *cli,
 	struct policy_handle 		connect_pol, domain_pol;
 	NTSTATUS status, result;
 	struct dom_sid                *sids;
-	uint32_t                     num_sids;
-	uint32			access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t                num_sids;
+	uint32_t		access_mask = MAXIMUM_ALLOWED_ACCESS;
 	int 			i;
 	struct lsa_SidArray sid_array;
 	struct samr_Ids alias_rids;
@@ -738,7 +738,7 @@ static NTSTATUS cmd_samr_query_useraliases(struct rpc_pipe_client *cli,
 	}
 
 	if (num_sids) {
-		sid_array.sids = TALLOC_ZERO_ARRAY(mem_ctx, struct lsa_SidPtr, num_sids);
+		sid_array.sids = talloc_zero_array(mem_ctx, struct lsa_SidPtr, num_sids);
 		if (sid_array.sids == NULL)
 			return NT_STATUS_NO_MEMORY;
 	} else {
@@ -801,8 +801,8 @@ static NTSTATUS cmd_samr_query_groupmem(struct rpc_pipe_client *cli,
 {
 	struct policy_handle connect_pol, domain_pol, group_pol;
 	NTSTATUS status, result;
-	uint32 group_rid;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t group_rid;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	int i;
 	unsigned int old_timeout;
 	struct samr_RidAttrArray *rids = NULL;
@@ -890,12 +890,13 @@ static NTSTATUS cmd_samr_enum_dom_users(struct rpc_pipe_client *cli,
 					TALLOC_CTX *mem_ctx,
 					int argc, const char **argv)
 {
-	struct policy_handle connect_pol, domain_pol;
+	struct policy_handle connect_pol;
+	struct policy_handle domain_pol = { 0, };
 	NTSTATUS status, result;
-	uint32 start_idx, num_dom_users, i;
+	uint32_t start_idx, num_dom_users, i;
 	struct samr_SamArray *dom_users = NULL;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
-	uint32 acb_mask = ACB_NORMAL;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t acb_mask = ACB_NORMAL;
 	uint32_t size = 0xffff;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
 
@@ -979,10 +980,11 @@ static NTSTATUS cmd_samr_enum_dom_groups(struct rpc_pipe_client *cli,
                                          TALLOC_CTX *mem_ctx,
                                          int argc, const char **argv)
 {
-	struct policy_handle connect_pol, domain_pol;
+	struct policy_handle connect_pol;
+	struct policy_handle domain_pol = { 0, };
 	NTSTATUS status, result;
-	uint32 start_idx, num_dom_groups, i;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t start_idx, num_dom_groups, i;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	struct samr_SamArray *dom_groups = NULL;
 	uint32_t size = 0xffff;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
@@ -1062,10 +1064,11 @@ static NTSTATUS cmd_samr_enum_als_groups(struct rpc_pipe_client *cli,
                                          TALLOC_CTX *mem_ctx,
                                          int argc, const char **argv)
 {
-	struct policy_handle connect_pol, domain_pol;
+	struct policy_handle connect_pol;
+	struct policy_handle domain_pol = { 0, };
 	NTSTATUS status, result;
-	uint32 start_idx, num_als_groups, i;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t start_idx, num_als_groups, i;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	struct samr_SamArray *als_groups = NULL;
 	uint32_t size = 0xffff;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
@@ -1146,8 +1149,8 @@ static NTSTATUS cmd_samr_enum_domains(struct rpc_pipe_client *cli,
 {
 	struct policy_handle connect_pol;
 	NTSTATUS status, result;
-	uint32 start_idx, size, num_entries, i;
-	uint32 access_mask = SEC_FLAG_MAXIMUM_ALLOWED;
+	uint32_t start_idx, size, num_entries, i;
+	uint32_t access_mask = SEC_FLAG_MAXIMUM_ALLOWED;
 	struct samr_SamArray *sam = NULL;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
 
@@ -1212,8 +1215,8 @@ static NTSTATUS cmd_samr_query_aliasmem(struct rpc_pipe_client *cli,
 {
 	struct policy_handle connect_pol, domain_pol, alias_pol;
 	NTSTATUS status, result;
-	uint32 alias_rid, i;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t alias_rid, i;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	struct lsa_SidArray sid_array;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
 
@@ -1276,10 +1279,10 @@ static NTSTATUS cmd_samr_query_aliasmem(struct rpc_pipe_client *cli,
 	}
 
 	for (i = 0; i < sid_array.num_sids; i++) {
-		fstring sid_str;
+		struct dom_sid_buf sid_str;
 
-		sid_to_fstring(sid_str, sid_array.sids[i].sid);
-		printf("\tsid:[%s]\n", sid_str);
+		printf("\tsid:[%s]\n",
+		       dom_sid_str_buf(sid_array.sids[i].sid, &sid_str));
 	}
 
 	dcerpc_samr_Close(b, mem_ctx, &alias_pol, &result);
@@ -1400,8 +1403,8 @@ static NTSTATUS cmd_samr_delete_alias(struct rpc_pipe_client *cli,
 {
 	struct policy_handle connect_pol, domain_pol, alias_pol;
 	NTSTATUS status, result;
-	uint32 alias_rid;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t alias_rid;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
 
 	if (argc != 3) {
@@ -1507,9 +1510,9 @@ static NTSTATUS cmd_samr_query_dispinfo_internal(struct rpc_pipe_client *cli,
 {
 	struct policy_handle connect_pol, domain_pol;
 	NTSTATUS status, result;
-	uint32 start_idx=0, max_entries=250, max_size = 0xffff, num_entries = 0, i;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
-	uint32 info_level = 1;
+	uint32_t start_idx=0, max_entries=250, max_size = 0xffff, num_entries = 0, i;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t info_level = 1;
 	union samr_DispInfo info;
 	int loop_count = 0;
 	bool got_params = False; /* Use get_query_dispinfo_params() or not? */
@@ -1712,8 +1715,8 @@ static NTSTATUS cmd_samr_query_dominfo(struct rpc_pipe_client *cli,
 {
 	struct policy_handle connect_pol, domain_pol;
 	NTSTATUS status, result;
-	uint32 switch_level = 2;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t switch_level = 2;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	union samr_DomainInfo *info = NULL;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
 
@@ -1827,9 +1830,9 @@ static NTSTATUS cmd_samr_create_dom_user(struct rpc_pipe_client *cli,
 	struct policy_handle connect_pol, domain_pol, user_pol;
 	NTSTATUS status, result;
 	struct lsa_String acct_name;
-	uint32 acb_info;
-	uint32 acct_flags, user_rid;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t acb_info;
+	uint32_t acct_flags, user_rid;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	uint32_t access_granted = 0;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
 
@@ -1916,7 +1919,7 @@ static NTSTATUS cmd_samr_create_dom_group(struct rpc_pipe_client *cli,
 	struct policy_handle connect_pol, domain_pol, group_pol;
 	NTSTATUS status, result;
 	struct lsa_String grp_name;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	uint32_t rid = 0;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
 
@@ -1993,7 +1996,7 @@ static NTSTATUS cmd_samr_create_dom_alias(struct rpc_pipe_client *cli,
 	struct policy_handle connect_pol, domain_pol, alias_pol;
 	NTSTATUS status, result;
 	struct lsa_String alias_name;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	uint32_t rid = 0;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
 
@@ -2071,7 +2074,7 @@ static NTSTATUS cmd_samr_lookup_names(struct rpc_pipe_client *cli,
 {
 	NTSTATUS status, result;
 	struct policy_handle connect_pol, domain_pol;
-	uint32 num_names;
+	uint32_t num_names;
 	struct samr_Ids rids, name_types;
 	int i;
 	struct lsa_String *names = NULL;
@@ -2106,7 +2109,7 @@ static NTSTATUS cmd_samr_lookup_names(struct rpc_pipe_client *cli,
 
 	num_names = argc - 2;
 
-	if ((names = TALLOC_ARRAY(mem_ctx, struct lsa_String, num_names)) == NULL) {
+	if ((names = talloc_array(mem_ctx, struct lsa_String, num_names)) == NULL) {
 		dcerpc_samr_Close(b, mem_ctx, &domain_pol, &result);
 		dcerpc_samr_Close(b, mem_ctx, &connect_pol, &result);
 		status = NT_STATUS_NO_MEMORY;
@@ -2194,7 +2197,7 @@ static NTSTATUS cmd_samr_lookup_rids(struct rpc_pipe_client *cli,
 
 	num_rids = argc - 2;
 
-	if ((rids = TALLOC_ARRAY(mem_ctx, uint32, num_rids)) == NULL) {
+	if ((rids = talloc_array(mem_ctx, uint32_t, num_rids)) == NULL) {
 		dcerpc_samr_Close(b, mem_ctx, &domain_pol, &result);
 		dcerpc_samr_Close(b, mem_ctx, &connect_pol, &result);
 		status = NT_STATUS_NO_MEMORY;
@@ -2248,7 +2251,7 @@ static NTSTATUS cmd_samr_delete_dom_group(struct rpc_pipe_client *cli,
 {
 	NTSTATUS status, result;
 	struct policy_handle connect_pol, domain_pol, group_pol;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
 
 	if ((argc < 2) || (argc > 3)) {
@@ -2359,7 +2362,7 @@ static NTSTATUS cmd_samr_delete_dom_user(struct rpc_pipe_client *cli,
 {
 	NTSTATUS status, result;
 	struct policy_handle connect_pol, domain_pol, user_pol;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
 
 	if ((argc < 2) || (argc > 3)) {
@@ -2471,8 +2474,8 @@ static NTSTATUS cmd_samr_query_sec_obj(struct rpc_pipe_client *cli,
 {
 	struct policy_handle connect_pol, domain_pol, user_pol, *pol;
 	NTSTATUS status, result;
-	uint32 sec_info = SECINFO_DACL;
-	uint32 user_rid = 0;
+	uint32_t sec_info = SECINFO_DACL;
+	uint32_t user_rid = 0;
 	TALLOC_CTX *ctx = NULL;
 	struct sec_desc_buf *sec_desc_buf=NULL;
 	bool domain = False;
@@ -2686,8 +2689,7 @@ static NTSTATUS cmd_samr_lookup_domain(struct rpc_pipe_client *cli,
 {
 	struct policy_handle connect_pol, domain_pol;
 	NTSTATUS status, result;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
-	fstring sid_string;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	struct lsa_String domain_name;
 	struct dom_sid *sid = NULL;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
@@ -2734,9 +2736,10 @@ static NTSTATUS cmd_samr_lookup_domain(struct rpc_pipe_client *cli,
 	}
 
 	if (NT_STATUS_IS_OK(result)) {
-		sid_to_fstring(sid_string, sid);
+		struct dom_sid_buf sid_str;
 		printf("SAMR_LOOKUP_DOMAIN: Domain Name: %s Domain SID: %s\n",
-		       argv[1], sid_string);
+		       argv[1],
+		       dom_sid_str_buf(sid, &sid_str));
 	}
 
 	dcerpc_samr_Close(b, mem_ctx, &domain_pol, &result);
@@ -2751,10 +2754,12 @@ static NTSTATUS cmd_samr_chgpasswd(struct rpc_pipe_client *cli,
 				   TALLOC_CTX *mem_ctx,
 				   int argc, const char **argv)
 {
-	struct policy_handle connect_pol, domain_pol, user_pol;
+	struct policy_handle connect_pol;
+	struct policy_handle domain_pol = { 0, };
+	struct policy_handle user_pol = { 0, };
 	NTSTATUS status, result;
 	const char *user, *oldpass, *newpass;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	struct samr_Ids rids, types;
 	struct lsa_String lsa_acct_name;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
@@ -2865,7 +2870,7 @@ static NTSTATUS cmd_samr_chgpasswd2(struct rpc_pipe_client *cli,
 	struct policy_handle connect_pol, domain_pol;
 	NTSTATUS status, result;
 	const char *user, *oldpass, *newpass;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
 
 	if (argc < 3) {
@@ -2929,7 +2934,7 @@ static NTSTATUS cmd_samr_chgpasswd3(struct rpc_pipe_client *cli,
 	struct policy_handle connect_pol, domain_pol;
 	NTSTATUS status, result;
 	const char *user, *oldpass, *newpass;
-	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+	uint32_t access_mask = MAXIMUM_ALLOWED_ACCESS;
 	struct samr_DomInfo1 *info = NULL;
 	struct userPwdChangeFailureInformation *reject = NULL;
 	struct dcerpc_binding_handle *b = cli->binding_handle;
@@ -3294,7 +3299,7 @@ static NTSTATUS cmd_samr_get_dispinfo_idx(struct rpc_pipe_client *cli,
 {
 	NTSTATUS status, result;
 	struct policy_handle connect_handle;
-	struct policy_handle domain_handle;
+	struct policy_handle domain_handle = { 0, };
 	uint16_t level = 1;
 	struct lsa_String name;
 	uint32_t idx = 0;
@@ -3366,40 +3371,40 @@ struct cmd_set samr_commands[] = {
 
 	{ "SAMR" },
 
-	{ "queryuser", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_user, 		NULL, &ndr_table_samr.syntax_id, NULL,	"Query user info",         "" },
-	{ "querygroup", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_group, 		NULL, &ndr_table_samr.syntax_id, NULL,	"Query group info",        "" },
-	{ "queryusergroups", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_usergroups, 	NULL, &ndr_table_samr.syntax_id, NULL,	"Query user groups",       "" },
-	{ "queryuseraliases", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_useraliases, 	NULL, &ndr_table_samr.syntax_id, NULL,	"Query user aliases",      "" },
-	{ "querygroupmem", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_groupmem, 	NULL, &ndr_table_samr.syntax_id, NULL,	"Query group membership",  "" },
-	{ "queryaliasmem", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_aliasmem, 	NULL, &ndr_table_samr.syntax_id, NULL,	"Query alias membership",  "" },
-	{ "queryaliasinfo", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_aliasinfo, 	NULL, &ndr_table_samr.syntax_id, NULL,	"Query alias info",       "" },
-	{ "deletealias", 	RPC_RTYPE_NTSTATUS, cmd_samr_delete_alias, 	NULL, &ndr_table_samr.syntax_id, NULL,	"Delete an alias",  "" },
-	{ "querydispinfo", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_dispinfo, 	NULL, &ndr_table_samr.syntax_id, NULL,	"Query display info",      "" },
-	{ "querydispinfo2", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_dispinfo2, 	NULL, &ndr_table_samr.syntax_id, NULL,	"Query display info",      "" },
-	{ "querydispinfo3", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_dispinfo3, 	NULL, &ndr_table_samr.syntax_id, NULL,	"Query display info",      "" },
-	{ "querydominfo", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_dominfo, 	NULL, &ndr_table_samr.syntax_id, NULL,	"Query domain info",       "" },
-	{ "enumdomusers",       RPC_RTYPE_NTSTATUS, cmd_samr_enum_dom_users,       NULL, &ndr_table_samr.syntax_id, NULL,	"Enumerate domain users", "" },
-	{ "enumdomgroups",      RPC_RTYPE_NTSTATUS, cmd_samr_enum_dom_groups,       NULL, &ndr_table_samr.syntax_id, NULL,	"Enumerate domain groups", "" },
-	{ "enumalsgroups",      RPC_RTYPE_NTSTATUS, cmd_samr_enum_als_groups,       NULL, &ndr_table_samr.syntax_id, NULL,	"Enumerate alias groups",  "" },
-	{ "enumdomains",        RPC_RTYPE_NTSTATUS, cmd_samr_enum_domains,          NULL, &ndr_table_samr.syntax_id, NULL,	"Enumerate domains",  "" },
+	{ "queryuser", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_user, 		NULL, &ndr_table_samr, NULL,	"Query user info",         "" },
+	{ "querygroup", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_group, 		NULL, &ndr_table_samr, NULL,	"Query group info",        "" },
+	{ "queryusergroups", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_usergroups, 	NULL, &ndr_table_samr, NULL,	"Query user groups",       "" },
+	{ "queryuseraliases", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_useraliases, 	NULL, &ndr_table_samr, NULL,	"Query user aliases",      "" },
+	{ "querygroupmem", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_groupmem, 	NULL, &ndr_table_samr, NULL,	"Query group membership",  "" },
+	{ "queryaliasmem", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_aliasmem, 	NULL, &ndr_table_samr, NULL,	"Query alias membership",  "" },
+	{ "queryaliasinfo", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_aliasinfo, 	NULL, &ndr_table_samr, NULL,	"Query alias info",       "" },
+	{ "deletealias", 	RPC_RTYPE_NTSTATUS, cmd_samr_delete_alias, 	NULL, &ndr_table_samr, NULL,	"Delete an alias",  "" },
+	{ "querydispinfo", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_dispinfo, 	NULL, &ndr_table_samr, NULL,	"Query display info",      "" },
+	{ "querydispinfo2", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_dispinfo2, 	NULL, &ndr_table_samr, NULL,	"Query display info",      "" },
+	{ "querydispinfo3", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_dispinfo3, 	NULL, &ndr_table_samr, NULL,	"Query display info",      "" },
+	{ "querydominfo", 	RPC_RTYPE_NTSTATUS, cmd_samr_query_dominfo, 	NULL, &ndr_table_samr, NULL,	"Query domain info",       "" },
+	{ "enumdomusers",       RPC_RTYPE_NTSTATUS, cmd_samr_enum_dom_users,       NULL, &ndr_table_samr, NULL,	"Enumerate domain users", "" },
+	{ "enumdomgroups",      RPC_RTYPE_NTSTATUS, cmd_samr_enum_dom_groups,       NULL, &ndr_table_samr, NULL,	"Enumerate domain groups", "" },
+	{ "enumalsgroups",      RPC_RTYPE_NTSTATUS, cmd_samr_enum_als_groups,       NULL, &ndr_table_samr, NULL,	"Enumerate alias groups",  "" },
+	{ "enumdomains",        RPC_RTYPE_NTSTATUS, cmd_samr_enum_domains,          NULL, &ndr_table_samr, NULL,	"Enumerate domains",  "" },
 
-	{ "createdomuser",      RPC_RTYPE_NTSTATUS, cmd_samr_create_dom_user,       NULL, &ndr_table_samr.syntax_id, NULL,	"Create domain user",      "" },
-	{ "createdomgroup",     RPC_RTYPE_NTSTATUS, cmd_samr_create_dom_group,      NULL, &ndr_table_samr.syntax_id, NULL,	"Create domain group",     "" },
-	{ "createdomalias",     RPC_RTYPE_NTSTATUS, cmd_samr_create_dom_alias,      NULL, &ndr_table_samr.syntax_id, NULL,	"Create domain alias",     "" },
-	{ "samlookupnames",     RPC_RTYPE_NTSTATUS, cmd_samr_lookup_names,          NULL, &ndr_table_samr.syntax_id, NULL,	"Look up names",           "" },
-	{ "samlookuprids",      RPC_RTYPE_NTSTATUS, cmd_samr_lookup_rids,           NULL, &ndr_table_samr.syntax_id, NULL,	"Look up names",           "" },
-	{ "deletedomgroup",     RPC_RTYPE_NTSTATUS, cmd_samr_delete_dom_group,      NULL, &ndr_table_samr.syntax_id, NULL,	"Delete domain group",     "" },
-	{ "deletedomuser",      RPC_RTYPE_NTSTATUS, cmd_samr_delete_dom_user,       NULL, &ndr_table_samr.syntax_id, NULL,	"Delete domain user",      "" },
-	{ "samquerysecobj",     RPC_RTYPE_NTSTATUS, cmd_samr_query_sec_obj,         NULL, &ndr_table_samr.syntax_id, NULL, "Query SAMR security object",   "" },
-	{ "getdompwinfo",       RPC_RTYPE_NTSTATUS, cmd_samr_get_dom_pwinfo,        NULL, &ndr_table_samr.syntax_id, NULL, "Retrieve domain password info", "" },
-	{ "getusrdompwinfo",    RPC_RTYPE_NTSTATUS, cmd_samr_get_usrdom_pwinfo,     NULL, &ndr_table_samr.syntax_id, NULL, "Retrieve user domain password info", "" },
+	{ "createdomuser",      RPC_RTYPE_NTSTATUS, cmd_samr_create_dom_user,       NULL, &ndr_table_samr, NULL,	"Create domain user",      "" },
+	{ "createdomgroup",     RPC_RTYPE_NTSTATUS, cmd_samr_create_dom_group,      NULL, &ndr_table_samr, NULL,	"Create domain group",     "" },
+	{ "createdomalias",     RPC_RTYPE_NTSTATUS, cmd_samr_create_dom_alias,      NULL, &ndr_table_samr, NULL,	"Create domain alias",     "" },
+	{ "samlookupnames",     RPC_RTYPE_NTSTATUS, cmd_samr_lookup_names,          NULL, &ndr_table_samr, NULL,	"Look up names",           "" },
+	{ "samlookuprids",      RPC_RTYPE_NTSTATUS, cmd_samr_lookup_rids,           NULL, &ndr_table_samr, NULL,	"Look up names",           "" },
+	{ "deletedomgroup",     RPC_RTYPE_NTSTATUS, cmd_samr_delete_dom_group,      NULL, &ndr_table_samr, NULL,	"Delete domain group",     "" },
+	{ "deletedomuser",      RPC_RTYPE_NTSTATUS, cmd_samr_delete_dom_user,       NULL, &ndr_table_samr, NULL,	"Delete domain user",      "" },
+	{ "samquerysecobj",     RPC_RTYPE_NTSTATUS, cmd_samr_query_sec_obj,         NULL, &ndr_table_samr, NULL, "Query SAMR security object",   "" },
+	{ "getdompwinfo",       RPC_RTYPE_NTSTATUS, cmd_samr_get_dom_pwinfo,        NULL, &ndr_table_samr, NULL, "Retrieve domain password info", "" },
+	{ "getusrdompwinfo",    RPC_RTYPE_NTSTATUS, cmd_samr_get_usrdom_pwinfo,     NULL, &ndr_table_samr, NULL, "Retrieve user domain password info", "" },
 
-	{ "lookupdomain",       RPC_RTYPE_NTSTATUS, cmd_samr_lookup_domain,         NULL, &ndr_table_samr.syntax_id, NULL, "Lookup Domain Name", "" },
-	{ "chgpasswd",          RPC_RTYPE_NTSTATUS, cmd_samr_chgpasswd,             NULL, &ndr_table_samr.syntax_id, NULL, "Change user password", "" },
-	{ "chgpasswd2",         RPC_RTYPE_NTSTATUS, cmd_samr_chgpasswd2,            NULL, &ndr_table_samr.syntax_id, NULL, "Change user password", "" },
-	{ "chgpasswd3",         RPC_RTYPE_NTSTATUS, cmd_samr_chgpasswd3,            NULL, &ndr_table_samr.syntax_id, NULL, "Change user password", "" },
-	{ "getdispinfoidx",     RPC_RTYPE_NTSTATUS, cmd_samr_get_dispinfo_idx,      NULL, &ndr_table_samr.syntax_id, NULL, "Get Display Information Index", "" },
-	{ "setuserinfo",        RPC_RTYPE_NTSTATUS, cmd_samr_setuserinfo,           NULL, &ndr_table_samr.syntax_id, NULL, "Set user info", "" },
-	{ "setuserinfo2",       RPC_RTYPE_NTSTATUS, cmd_samr_setuserinfo2,          NULL, &ndr_table_samr.syntax_id, NULL, "Set user info2", "" },
+	{ "lookupdomain",       RPC_RTYPE_NTSTATUS, cmd_samr_lookup_domain,         NULL, &ndr_table_samr, NULL, "Lookup Domain Name", "" },
+	{ "chgpasswd",          RPC_RTYPE_NTSTATUS, cmd_samr_chgpasswd,             NULL, &ndr_table_samr, NULL, "Change user password", "" },
+	{ "chgpasswd2",         RPC_RTYPE_NTSTATUS, cmd_samr_chgpasswd2,            NULL, &ndr_table_samr, NULL, "Change user password", "" },
+	{ "chgpasswd3",         RPC_RTYPE_NTSTATUS, cmd_samr_chgpasswd3,            NULL, &ndr_table_samr, NULL, "Change user password", "" },
+	{ "getdispinfoidx",     RPC_RTYPE_NTSTATUS, cmd_samr_get_dispinfo_idx,      NULL, &ndr_table_samr, NULL, "Get Display Information Index", "" },
+	{ "setuserinfo",        RPC_RTYPE_NTSTATUS, cmd_samr_setuserinfo,           NULL, &ndr_table_samr, NULL, "Set user info", "" },
+	{ "setuserinfo2",       RPC_RTYPE_NTSTATUS, cmd_samr_setuserinfo2,          NULL, &ndr_table_samr, NULL, "Set user info2", "" },
 	{ NULL }
 };
