@@ -142,7 +142,7 @@ static int net_sam_set_workstations(struct net_context *c, int argc,
 
 static int net_sam_set_userflag(struct net_context *c, int argc,
 				const char **argv, const char *field,
-				uint16 flag)
+				uint16_t flag)
 {
 	struct samu *sam_acct = NULL;
 	struct dom_sid sid;
@@ -480,8 +480,8 @@ static int net_sam_set(struct net_context *c, int argc, const char **argv)
 static int net_sam_policy_set(struct net_context *c, int argc, const char **argv)
 {
 	const char *account_policy = NULL;
-	uint32 value = 0;
-	uint32 old_value = 0;
+	uint32_t value = 0;
+	uint32_t old_value = 0;
 	enum pdb_policy_type field;
 	char *endptr;
 
@@ -550,7 +550,7 @@ static int net_sam_policy_set(struct net_context *c, int argc, const char **argv
 static int net_sam_policy_show(struct net_context *c, int argc, const char **argv)
 {
 	const char *account_policy = NULL;
-        uint32 old_value;
+        uint32_t old_value;
         enum pdb_policy_type field;
 
         if (argc != 1 || c->display_usage) {
@@ -693,13 +693,15 @@ static int net_sam_rights_list(struct net_context *c, int argc,
 		for (i=0; i<num_sids; i++) {
 			const char *dom, *name;
 			enum lsa_SidType type;
+			struct dom_sid_buf buf;
 
 			if (lookup_sid(talloc_tos(), &sids[i], &dom, &name,
 				       &type)) {
 				d_printf("%s\\%s\n", dom, name);
 			}
 			else {
-				d_printf("%s\n", sid_string_tos(&sids[i]));
+				d_printf("%s\n",
+					 dom_sid_str_buf(&sids[i], &buf));
 			}
 		}
 		return 0;
@@ -825,7 +827,7 @@ static int net_sam_rights(struct net_context *c, int argc, const char **argv)
 static NTSTATUS map_unix_group(const struct group *grp, GROUP_MAP *map)
 {
 	const char *dom, *name;
-	uint32 rid;
+	uint32_t rid;
 
 	if (pdb_getgrgid(map, grp->gr_gid)) {
 		return NT_STATUS_GROUP_EXISTS;
@@ -871,6 +873,7 @@ static int net_sam_mapunixgroup(struct net_context *c, int argc, const char **ar
 	NTSTATUS status;
 	GROUP_MAP *map;
 	struct group *grp;
+	struct dom_sid_buf buf;
 
 	if (argc != 1 || c->display_usage) {
 		d_fprintf(stderr, "%s\n%s",
@@ -900,7 +903,7 @@ static int net_sam_mapunixgroup(struct net_context *c, int argc, const char **ar
 	}
 
 	d_printf(_("Mapped unix group %s to SID %s\n"), argv[0],
-		 sid_string_tos(&map->sid));
+		 dom_sid_str_buf(&map->sid, &buf));
 
 	TALLOC_FREE(map);
 	return 0;
@@ -970,7 +973,7 @@ static int net_sam_createdomaingroup(struct net_context *c, int argc,
 				     const char **argv)
 {
 	NTSTATUS status;
-	uint32 rid;
+	uint32_t rid;
 
 	if (argc != 1 || c->display_usage) {
 		d_fprintf(stderr, "%s\n%s",
@@ -1046,7 +1049,7 @@ static int net_sam_deletedomaingroup(struct net_context *c, int argc,
 static int net_sam_createlocalgroup(struct net_context *c, int argc, const char **argv)
 {
 	NTSTATUS status;
-	uint32 rid;
+	uint32_t rid;
 
 	if (argc != 1 || c->display_usage) {
 		d_fprintf(stderr, "%s\n%s",
@@ -1124,7 +1127,7 @@ static int net_sam_deletelocalgroup(struct net_context *c, int argc, const char
 static int net_sam_createbuiltingroup(struct net_context *c, int argc, const char **argv)
 {
 	NTSTATUS status;
-	uint32 rid;
+	uint32_t rid;
 	enum lsa_SidType type;
 	fstring groupname;
 	struct dom_sid sid;
@@ -1328,8 +1331,11 @@ static int net_sam_delmem(struct net_context *c, int argc, const char **argv)
 		d_printf(_("Deleted %s\\%s from %s\\%s\n"),
 			 memberdomain, membername, groupdomain, groupname);
 	} else {
+		struct dom_sid_buf buf;
 		d_printf(_("Deleted %s from %s\\%s\n"),
-			 sid_string_tos(&member), groupdomain, groupname);
+			 dom_sid_str_buf(&member, &buf),
+			 groupdomain,
+			 groupname);
 	}
 
 	return 0;
@@ -1406,7 +1412,9 @@ static int net_sam_listmem(struct net_context *c, int argc, const char **argv)
 		if (lookup_sid(talloc_tos(), &members[i], &dom, &name, NULL)) {
 			d_printf(" %s\\%s\n", dom, name);
 		} else {
-			d_printf(" %s\n", sid_string_tos(&members[i]));
+			struct dom_sid_buf buf;
+			d_printf(" %s\n",
+				 dom_sid_str_buf(&members[i], &buf));
 		}
 	}
 
@@ -1555,6 +1563,7 @@ static int net_sam_list(struct net_context *c, int argc, const char **argv)
 static int net_sam_show(struct net_context *c, int argc, const char **argv)
 {
 	struct dom_sid sid;
+	struct dom_sid_buf buf;
 	enum lsa_SidType type;
 	const char *dom, *name;
 
@@ -1572,7 +1581,7 @@ static int net_sam_show(struct net_context *c, int argc, const char **argv)
 	}
 
 	d_printf(_("%s\\%s is a %s with SID %s\n"), dom, name,
-		 sid_type_lookup(type), sid_string_tos(&sid));
+		 sid_type_lookup(type), dom_sid_str_buf(&sid, &buf));
 
 	return 0;
 }
@@ -1682,6 +1691,7 @@ static int net_sam_provision(struct net_context *c, int argc, const char **argv)
 	}
 
 	if (!pdb_getgrsid(gmap, gsid)) {
+		struct dom_sid_buf gsid_str;
 		LDAPMod **mods = NULL;
 		char *dn;
 		char *uname;
@@ -1726,7 +1736,7 @@ static int net_sam_provision(struct net_context *c, int argc, const char **argv)
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "displayName", wname);
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "gidNumber", gidstr);
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "sambaSid",
-				sid_string_talloc(tc, &gsid));
+				dom_sid_str_buf(&gsid, &gsid_str));
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "sambaGroupType", gtype);
 
 		smbldap_talloc_autofree_ldapmod(tc, mods);
@@ -1759,6 +1769,7 @@ domu_done:
 	sid_compose(&gsid, get_global_sam_sid(), DOMAIN_RID_ADMINS);
 
 	if (!pdb_getgrsid(gmap, gsid)) {
+		struct dom_sid_buf gsid_str;
 		LDAPMod **mods = NULL;
 		char *dn;
 		char *uname;
@@ -1803,7 +1814,7 @@ domu_done:
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "displayName", wname);
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "gidNumber", gidstr);
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "sambaSid",
-				sid_string_talloc(tc, &gsid));
+				dom_sid_str_buf(&gsid, &gsid_str));
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "sambaGroupType", gtype);
 
 		smbldap_talloc_autofree_ldapmod(tc, mods);
@@ -1842,6 +1853,7 @@ doma_done:
 	if (!pdb_getsampwnam(samuser, "Administrator")) {
 		LDAPMod **mods = NULL;
 		struct dom_sid sid;
+		struct dom_sid_buf sid_str;
 		char *dn;
 		char *name;
 		char *uidstr;
@@ -1921,7 +1933,7 @@ doma_done:
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "homeDirectory", dir);
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "loginShell", shell);
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "sambaSID",
-				sid_string_talloc(tc, &sid));
+				dom_sid_str_buf(&sid, &sid_str));
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "sambaAcctFlags",
 				pdb_encode_acct_ctrl(ACB_NORMAL|ACB_DISABLED,
 				NEW_PW_FORMAT_SPACE_PADDED_LEN));
@@ -1957,6 +1969,7 @@ doma_done:
 	if (!pdb_getsampwnam(samuser, lp_guest_account())) {
 		LDAPMod **mods = NULL;
 		struct dom_sid sid;
+		struct dom_sid_buf sid_str;
 		char *dn;
 		char *uidstr;
 		char *gidstr;
@@ -2033,7 +2046,7 @@ doma_done:
 			smbldap_set_mod(&mods, LDAP_MOD_ADD, "loginShell", pwd->pw_shell);
 		}
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "sambaSID",
-				sid_string_talloc(tc, &sid));
+				dom_sid_str_buf(&sid, &sid_str));
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "sambaAcctFlags",
 				pdb_encode_acct_ctrl(ACB_NORMAL|ACB_DISABLED,
 				NEW_PW_FORMAT_SPACE_PADDED_LEN));
@@ -2074,6 +2087,7 @@ doma_done:
 	}
 
 	if (!pdb_getgrgid(gmap, pwd->pw_gid)) {
+		struct dom_sid_buf gsid_str;
 		LDAPMod **mods = NULL;
 		char *dn;
 		char *uname;
@@ -2109,7 +2123,7 @@ doma_done:
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "displayName", wname);
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "gidNumber", gidstr);
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "sambaSid",
-				sid_string_talloc(tc, &gsid));
+				dom_sid_str_buf(&gsid, &gsid_str));
 		smbldap_set_mod(&mods, LDAP_MOD_ADD, "sambaGroupType", gtype);
 
 		smbldap_talloc_autofree_ldapmod(tc, mods);
