@@ -100,7 +100,6 @@ WERROR dns_verify_tsig(struct dns_server *dns,
 	WERROR werror;
 	NTSTATUS status;
 	enum ndr_err_code ndr_err;
-	bool found_tsig = false;
 	uint16_t i, arcount = 0;
 	DATA_BLOB tsig_blob, fake_tsig_blob, sig;
 	uint8_t *buffer = NULL;
@@ -113,17 +112,17 @@ WERROR dns_verify_tsig(struct dns_server *dns,
 	/* Find the first TSIG record in the additional records */
 	for (i=0; i < packet->arcount; i++) {
 		if (packet->additional[i].rr_type == DNS_QTYPE_TSIG) {
-			found_tsig = true;
 			break;
 		}
 	}
 
-	if (!found_tsig) {
+	if (i == packet->arcount) {
+		/* no TSIG around */
 		return WERR_OK;
 	}
 
 	/* The TSIG record needs to be the last additional record */
-	if (found_tsig && i + 1 != packet->arcount) {
+	if (i + 1 != packet->arcount) {
 		DEBUG(1, ("TSIG record not the last additional record!\n"));
 		return DNS_ERR(FORMAT_ERROR);
 	}
@@ -133,7 +132,7 @@ WERROR dns_verify_tsig(struct dns_server *dns,
 
 	state->tsig = talloc_zero(state->mem_ctx, struct dns_res_rec);
 	if (state->tsig == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	werror = dns_copy_tsig(state->tsig, &packet->additional[i],
@@ -154,7 +153,7 @@ WERROR dns_verify_tsig(struct dns_server *dns,
 		state->key_name = talloc_strdup(state->mem_ctx,
 						state->tsig->name);
 		if (state->key_name == NULL) {
-			return WERR_NOMEM;
+			return WERR_NOT_ENOUGH_MEMORY;
 		}
 		state->tsig_error = DNS_RCODE_BADKEY;
 		return DNS_ERR(NOTAUTH);
@@ -167,24 +166,24 @@ WERROR dns_verify_tsig(struct dns_server *dns,
 	 */
 	state->key_name = talloc_strdup(state->mem_ctx, tkey->name);
 	if (state->key_name == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	/* FIXME: check TSIG here */
 	if (check_rec == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	/* first build and verify check packet */
 	check_rec->name = talloc_strdup(check_rec, tkey->name);
 	if (check_rec->name == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 	check_rec->rr_class = DNS_QCLASS_ANY;
 	check_rec->ttl = 0;
 	check_rec->algorithm_name = talloc_strdup(check_rec, tkey->algorithm);
 	if (check_rec->algorithm_name == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 	check_rec->time_prefix = 0;
 	check_rec->time = state->tsig->rdata.tsig_record.time;
@@ -215,7 +214,7 @@ WERROR dns_verify_tsig(struct dns_server *dns,
 	buffer_len = packet_len + fake_tsig_blob.length;
 	buffer = talloc_zero_array(mem_ctx, uint8_t, buffer_len);
 	if (buffer == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	memcpy(buffer, in->data, packet_len);
@@ -224,7 +223,7 @@ WERROR dns_verify_tsig(struct dns_server *dns,
 	sig.length = state->tsig->rdata.tsig_record.mac_size;
 	sig.data = talloc_memdup(mem_ctx, state->tsig->rdata.tsig_record.mac, sig.length);
 	if (sig.data == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	/* Now we also need to count down the additional record counter */
@@ -266,19 +265,19 @@ static WERROR dns_tsig_compute_mac(TALLOC_CTX *mem_ctx,
 	size_t mac_size = 0;
 
 	if (check_rec == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	/* first build and verify check packet */
 	check_rec->name = talloc_strdup(check_rec, tkey->name);
 	if (check_rec->name == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 	check_rec->rr_class = DNS_QCLASS_ANY;
 	check_rec->ttl = 0;
 	check_rec->algorithm_name = talloc_strdup(check_rec, tkey->algorithm);
 	if (check_rec->algorithm_name == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 	check_rec->time_prefix = 0;
 	check_rec->time = current_time;
@@ -311,16 +310,16 @@ static WERROR dns_tsig_compute_mac(TALLOC_CTX *mem_ctx,
 
 	buffer_len += packet_blob.length;
 	if (buffer_len < packet_blob.length) {
-		return WERR_INVALID_PARAM;
+		return WERR_INVALID_PARAMETER;
 	}
 	buffer_len += tsig_blob.length;
 	if (buffer_len < tsig_blob.length) {
-		return WERR_INVALID_PARAM;
+		return WERR_INVALID_PARAMETER;
 	}
 
 	buffer = talloc_zero_array(mem_ctx, uint8_t, buffer_len);
 	if (buffer == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	p = buffer;
@@ -368,7 +367,7 @@ WERROR dns_sign_tsig(struct dns_server *dns,
 
 	tsig = talloc_zero(mem_ctx, struct dns_res_rec);
 	if (tsig == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	if (state->tsig_error == DNS_RCODE_OK) {
@@ -387,13 +386,16 @@ WERROR dns_sign_tsig(struct dns_server *dns,
 
 	tsig->name = talloc_strdup(tsig, state->key_name);
 	if (tsig->name == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 	tsig->rr_class = DNS_QCLASS_ANY;
 	tsig->rr_type = DNS_QTYPE_TSIG;
 	tsig->ttl = 0;
 	tsig->length = UINT16_MAX;
 	tsig->rdata.tsig_record.algorithm_name = talloc_strdup(tsig, "gss-tsig");
+	if (tsig->rdata.tsig_record.algorithm_name == NULL) {
+		return WERR_NOT_ENOUGH_MEMORY;
+	}
 	tsig->rdata.tsig_record.time_prefix = 0;
 	tsig->rdata.tsig_record.time = current_time;
 	tsig->rdata.tsig_record.fudge = 300;
@@ -404,19 +406,22 @@ WERROR dns_sign_tsig(struct dns_server *dns,
 	if (sig.length > 0) {
 		tsig->rdata.tsig_record.mac_size = sig.length;
 		tsig->rdata.tsig_record.mac = talloc_memdup(tsig, sig.data, sig.length);
+		if (tsig->rdata.tsig_record.mac == NULL) {
+			return WERR_NOT_ENOUGH_MEMORY;
+		}
 	}
 
 	if (packet->arcount == 0) {
 		packet->additional = talloc_zero(mem_ctx, struct dns_res_rec);
 		if (packet->additional == NULL) {
-			return WERR_NOMEM;
+			return WERR_NOT_ENOUGH_MEMORY;
 		}
 	}
 	packet->additional = talloc_realloc(mem_ctx, packet->additional,
 					    struct dns_res_rec,
 					    packet->arcount + 1);
 	if (packet->additional == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	werror = dns_copy_tsig(mem_ctx, tsig,
