@@ -402,7 +402,7 @@ static int extended_callback(struct ldb_request *req, struct ldb_reply *ares,
 	struct ldb_message *msg;
 	struct extended_dn_out_private *p;
 	struct ldb_context *ldb;
-	bool have_reveal_control=false, checked_reveal_control=false;
+	bool have_reveal_control=false;
 
 	ac = talloc_get_type(req->context, struct extended_search_context);
 	p = talloc_get_type(ldb_module_get_private(ac->module), struct extended_dn_out_private);
@@ -473,11 +473,8 @@ static int extended_callback(struct ldb_request *req, struct ldb_reply *ares,
 		}
 	}
 
-	if (!checked_reveal_control) {
-		have_reveal_control =
-			ldb_request_get_control(req, LDB_CONTROL_REVEAL_INTERNALS) != NULL;
-		checked_reveal_control = true;
-	}
+	have_reveal_control =
+		dsdb_request_has_control(req, LDB_CONTROL_REVEAL_INTERNALS);
 
 	/* 
 	 * Shortcut for repl_meta_data.  We asked for the data
@@ -498,7 +495,7 @@ static int extended_callback(struct ldb_request *req, struct ldb_reply *ares,
 			continue;
 		}
 
-		if (p->normalise) {
+		if (p && p->normalise) {
 			/* If we are also in 'normalise' mode, then
 			 * fix the attribute names to be in the
 			 * correct case */
@@ -548,9 +545,9 @@ static int extended_callback(struct ldb_request *req, struct ldb_reply *ares,
 			}
 
 
-			dsdb_dn = dsdb_dn_parse(msg, ldb, plain_dn, attribute->syntax->ldap_oid);
+			dsdb_dn = dsdb_dn_parse_trusted(msg, ldb, plain_dn, attribute->syntax->ldap_oid);
 
-			if (!dsdb_dn || !ldb_dn_validate(dsdb_dn->dn)) {
+			if (!dsdb_dn) {
 				ldb_asprintf_errstring(ldb, 
 						       "could not parse %.*s in %s on %s as a %s DN", 
 						       (int)plain_dn->length, plain_dn->data,
@@ -572,13 +569,6 @@ static int extended_callback(struct ldb_request *req, struct ldb_reply *ares,
 				talloc_free(hex_string);
 			}
 
-			/* don't let users see the internal extended
-			   GUID components */
-			if (!have_reveal_control) {
-				const char *accept[] = { "GUID", "SID", NULL };
-				ldb_dn_extended_filter(dn, accept);
-			}
-
 			if (p->normalise) {
 				ret = dsdb_fix_dn_rdncase(ldb, dn);
 				if (ret != LDB_SUCCESS) {
@@ -633,6 +623,21 @@ static int extended_callback(struct ldb_request *req, struct ldb_reply *ares,
 			}
 			
 			if (make_extended_dn) {
+				if (!ldb_dn_validate(dsdb_dn->dn)) {
+					ldb_asprintf_errstring(ldb, 
+							       "could not parse %.*s in %s on %s as a %s DN", 
+							       (int)plain_dn->length, plain_dn->data,
+							       msg->elements[i].name, ldb_dn_get_linearized(msg->dn),
+							       attribute->syntax->ldap_oid);
+					talloc_free(dsdb_dn);
+					return ldb_module_done(ac->req, NULL, NULL, LDB_ERR_INVALID_DN_SYNTAX);
+				}
+				/* don't let users see the internal extended
+				   GUID components */
+				if (!have_reveal_control) {
+					const char *accept[] = { "GUID", "SID", NULL };
+					ldb_dn_extended_filter(dn, accept);
+				}
 				dn_str = dsdb_dn_get_extended_linearized(msg->elements[i].values,
 									 dsdb_dn, ac->extended_type);
 			} else {
@@ -743,7 +748,7 @@ static int extended_dn_out_search(struct ldb_module *module, struct ldb_request
 			ac->extended_type = 0;
 		}
 
-		/* check if attrs only is specified, in that case check wether we need to modify them */
+		/* check if attrs only is specified, in that case check whether we need to modify them */
 		if (req->op.search.attrs && !is_attr_in_list(req->op.search.attrs, "*")) {
 			if (! is_attr_in_list(req->op.search.attrs, "objectGUID")) {
 				ac->remove_guid = true;
