@@ -994,7 +994,6 @@ static int get_pso_count(struct ldb_module *module, TALLOC_CTX *mem_ctx,
 	struct ldb_result *res = NULL;
 	struct ldb_context *ldb = ldb_module_get_ctx(module);
 
-	*pso_count = 0;
 	domain_dn = ldb_get_default_basedn(ldb);
 	psc_dn = ldb_dn_new_fmt(mem_ctx, ldb,
 			        "CN=Password Settings Container,CN=System,%s",
@@ -1008,17 +1007,6 @@ static int get_pso_count(struct ldb_module *module, TALLOC_CTX *mem_ctx,
 				 LDB_SCOPE_ONELEVEL, attrs,
 				 DSDB_FLAG_NEXT_MODULE, parent,
 				 "(objectClass=msDS-PasswordSettings)");
-
-	/*
-	 * Just ignore PSOs if the container doesn't exist. This is a weird
-	 * corner-case where the AD DB was created from a pre-2008 base schema,
-	 * and then the FL was manually upgraded.
-	 */
-	if (ret == LDB_ERR_NO_SUCH_OBJECT) {
-		DBG_NOTICE("No Password Settings Container exists\n");
-		return LDB_SUCCESS;
-	}
-
 	if (ret != LDB_SUCCESS) {
 		return ret;
 	}
@@ -1083,13 +1071,12 @@ static int pso_search_by_sids(struct ldb_module *module, TALLOC_CTX *mem_ctx,
 	sid_filter = talloc_strdup(mem_ctx, "");
 
 	for (i = 0; sid_filter && i < num_sids; i++) {
-		char sid_buf[DOM_SID_STR_BUFLEN] = {0,};
+		struct dom_sid_buf sid_buf;
 
-		dom_sid_string_buf(&sid_array[i], sid_buf, sizeof(sid_buf));
-
-		sid_filter = talloc_asprintf_append(sid_filter,
-						    "(msDS-PSOAppliesTo=<SID=%s>)",
-						    sid_buf);
+		sid_filter = talloc_asprintf_append(
+			sid_filter,
+			"(msDS-PSOAppliesTo=<SID=%s>)",
+			dom_sid_str_buf(&sid_array[i], &sid_buf));
 	}
 
 	if (sid_filter == NULL) {
