@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 #
 # Tests various schema replication scenarios
@@ -27,6 +27,7 @@
 #  PYTHONPATH="$PYTHONPATH:$samba4srcdir/torture/drs/python" $SUBUNITRUN replica_sync -U"$DOMAIN/$DC_USERNAME"%"$DC_PASSWORD"
 #
 
+from __future__ import print_function
 import drs_base
 import samba.tests
 import time
@@ -35,6 +36,7 @@ import ldb
 from ldb import (
     SCOPE_BASE, LdbError, ERR_NO_SUCH_OBJECT)
 
+
 class DrsReplicaSyncTestCase(drs_base.DrsBaseTestCase):
     """Intended as a black box test case for DsReplicaSync
        implementation. It should test the behavior of this
@@ -42,6 +44,11 @@ class DrsReplicaSyncTestCase(drs_base.DrsBaseTestCase):
 
     def setUp(self):
         super(DrsReplicaSyncTestCase, self).setUp()
+
+        # This OU avoids this test conflicting with anything
+        # that may already be in the DB
+        self.top_ou = samba.tests.create_test_ou(self.ldb_dc1,
+                                                 "replica_sync")
         self._net_drs_replicate(DC=self.dnsname_dc2, fromDC=self.dnsname_dc1, forced=True)
         self._net_drs_replicate(DC=self.dnsname_dc1, fromDC=self.dnsname_dc2, forced=True)
         self.ou1 = None
@@ -50,6 +57,7 @@ class DrsReplicaSyncTestCase(drs_base.DrsBaseTestCase):
     def tearDown(self):
         self._cleanup_object(self.ou1)
         self._cleanup_object(self.ou2)
+        self._cleanup_dn(self.top_ou)
 
         # re-enable replication
         self._enable_inbound_repl(self.dnsname_dc1)
@@ -57,17 +65,22 @@ class DrsReplicaSyncTestCase(drs_base.DrsBaseTestCase):
 
         super(DrsReplicaSyncTestCase, self).tearDown()
 
+    def _cleanup_dn(self, dn):
+        try:
+            self.ldb_dc2.delete(dn, ["tree_delete:1"])
+        except LdbError as e:
+            (num, _) = e.args
+            self.assertEquals(num, ERR_NO_SUCH_OBJECT)
+        try:
+            self.ldb_dc1.delete(dn, ["tree_delete:1"])
+        except LdbError as e1:
+            (num, _) = e1.args
+            self.assertEquals(num, ERR_NO_SUCH_OBJECT)
+
     def _cleanup_object(self, guid):
         """Cleans up a test object, if it still exists"""
         if guid is not None:
-            try:
-                self.ldb_dc2.delete('<GUID=%s>' % guid, ["tree_delete:1"])
-            except LdbError, (num, _):
-                self.assertEquals(num, ERR_NO_SUCH_OBJECT)
-            try:
-                self.ldb_dc1.delete('<GUID=%s>' % guid, ["tree_delete:1"])
-            except LdbError, (num, _):
-                self.assertEquals(num, ERR_NO_SUCH_OBJECT)
+            self._cleanup_dn('<GUID=%s>' % guid)
 
     def test_ReplEnabled(self):
         """Tests we can replicate when replication is enabled"""
@@ -109,9 +122,9 @@ class DrsReplicaSyncTestCase(drs_base.DrsBaseTestCase):
         ldif = """
 dn: %s,%s
 objectClass: organizationalUnit
-""" % (name, self.domain_dn)
+""" % (name, self.top_ou)
         samdb.add_ldif(ldif)
-        res = samdb.search(base="%s,%s" % (name, self.domain_dn),
+        res = samdb.search(base="%s,%s" % (name, self.top_ou),
                            scope=SCOPE_BASE, attrs=["objectGUID"])
         return self._GUID_string(res[0]["objectGUID"][0])
 
@@ -125,9 +138,9 @@ objectClass: organizationalUnit
         # Deleted Object base DN
         dodn = self._deleted_objects_dn(sam_ldb)
         # now check properties of the user
-        name_cur  = ou_cur["ou"][0]
-        self.assertEquals(ou_cur["isDeleted"][0],"TRUE")
-        self.assertTrue(not("objectCategory" in ou_cur))
+        name_cur = ou_cur["ou"][0]
+        self.assertEquals(ou_cur["isDeleted"][0], b"TRUE")
+        self.assertTrue(not(b"objectCategory" in ou_cur))
         self.assertTrue(dodn in str(ou_cur["dn"]),
                         "OU %s is deleted but it is not located under %s!" % (name_cur, dodn))
 
@@ -150,11 +163,11 @@ objectClass: organizationalUnit
 
         # Check that DC2 got the DC1 object, and OU1 was make into conflict
         res1 = self.ldb_dc2.search(base="<GUID=%s>" % self.ou1,
-                                  scope=SCOPE_BASE, attrs=["name"])
+                                   scope=SCOPE_BASE, attrs=["name"])
         res2 = self.ldb_dc2.search(base="<GUID=%s>" % self.ou2,
-                                  scope=SCOPE_BASE, attrs=["name"])
-        print res1[0]["name"][0]
-        print res2[0]["name"][0]
+                                   scope=SCOPE_BASE, attrs=["name"])
+        print(res1[0]["name"][0])
+        print(res2[0]["name"][0])
         self.assertFalse('CNF:%s' % self.ou2 in str(res2[0]["name"][0]))
         self.assertTrue('CNF:%s' % self.ou1 in str(res1[0]["name"][0]))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc2, self.domain_dn) not in str(res1[0].dn))
@@ -190,11 +203,11 @@ objectClass: organizationalUnit
 
         # Check that DC2 got the DC1 object, and OU1 was make into conflict
         res1 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou1,
-                                  scope=SCOPE_BASE, attrs=["name"])
+                                   scope=SCOPE_BASE, attrs=["name"])
         res2 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou2,
-                                  scope=SCOPE_BASE, attrs=["name"])
-        print res1[0]["name"][0]
-        print res2[0]["name"][0]
+                                   scope=SCOPE_BASE, attrs=["name"])
+        print(res1[0]["name"][0])
+        print(res2[0]["name"][0])
         self.assertTrue('CNF:%s' % self.ou1 in str(res1[0]["name"][0]))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res1[0].dn))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res2[0].dn))
@@ -229,11 +242,11 @@ objectClass: organizationalUnit
 
         # Check that DC2 got the DC1 object, and OU2 was make into conflict
         res1 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou1,
-                                  scope=SCOPE_BASE, attrs=["name"])
+                                   scope=SCOPE_BASE, attrs=["name"])
         res2 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou2,
-                                  scope=SCOPE_BASE, attrs=["name"])
-        print res1[0]["name"][0]
-        print res2[0]["name"][0]
+                                   scope=SCOPE_BASE, attrs=["name"])
+        print(res1[0]["name"][0])
+        print(res2[0]["name"][0])
         self.assertTrue('CNF:%s' % self.ou2 in str(res2[0]["name"][0]), "Got %s for %s" % (str(res2[0]["name"][0]), self.ou2))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res1[0].dn))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res2[0].dn))
@@ -271,11 +284,11 @@ objectClass: organizationalUnit
 
         # Check that DC2 got the DC1 object, and SELF.OU1 was make into conflict
         res1 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou1,
-                                  scope=SCOPE_BASE, attrs=["name"])
+                                   scope=SCOPE_BASE, attrs=["name"])
         res2 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou2,
-                                  scope=SCOPE_BASE, attrs=["name"])
-        print res1[0]["name"][0]
-        print res2[0]["name"][0]
+                                   scope=SCOPE_BASE, attrs=["name"])
+        print(res1[0]["name"][0])
+        print(res2[0]["name"][0])
         self.assertTrue('CNF:%s' % self.ou1 in str(res1[0]["name"][0]))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res1[0].dn))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res2[0].dn))
@@ -308,7 +321,7 @@ objectClass: organizationalUnit
 
         # Create an OU and rename it on DC1
         self.ou1 = self._create_ou(self.ldb_dc1, "OU=Test Remote Rename Conflict orig")
-        self.ldb_dc1.rename("<GUID=%s>" % self.ou1, "OU=Test Remote Rename Conflict,%s" % self.domain_dn)
+        self.ldb_dc1.rename("<GUID=%s>" % self.ou1, "OU=Test Remote Rename Conflict,%s" % self.top_ou)
 
         # We have to sleep to ensure that the two objects have different timestamps
         time.sleep(1)
@@ -320,11 +333,11 @@ objectClass: organizationalUnit
 
         # Check that DC2 got the DC1 object, and SELF.OU1 was made into conflict
         res1 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou1,
-                                  scope=SCOPE_BASE, attrs=["name"])
+                                   scope=SCOPE_BASE, attrs=["name"])
         res2 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou2,
-                                  scope=SCOPE_BASE, attrs=["name"])
-        print res1[0]["name"][0]
-        print res2[0]["name"][0]
+                                   scope=SCOPE_BASE, attrs=["name"])
+        print(res1[0]["name"][0])
+        print(res2[0]["name"][0])
         self.assertTrue('CNF:%s' % self.ou1 in str(res1[0]["name"][0]))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res1[0].dn))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res2[0].dn))
@@ -350,7 +363,7 @@ objectClass: organizationalUnit
 
         # Create conflicting objects on DC1 and DC2, where the DC2 object has been renamed
         self.ou2 = self._create_ou(self.ldb_dc2, "OU=Test Rename Local Conflict orig")
-        self.ldb_dc2.rename("<GUID=%s>" % self.ou2, "OU=Test Rename Local Conflict,%s" % self.domain_dn)
+        self.ldb_dc2.rename("<GUID=%s>" % self.ou2, "OU=Test Rename Local Conflict,%s" % self.top_ou)
         # We have to sleep to ensure that the two objects have different timestamps
         time.sleep(1)
         self.ou1 = self._create_ou(self.ldb_dc1, "OU=Test Rename Local Conflict")
@@ -359,11 +372,11 @@ objectClass: organizationalUnit
 
         # Check that DC2 got the DC1 object, and OU2 was made into conflict
         res1 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou1,
-                                  scope=SCOPE_BASE, attrs=["name"])
+                                   scope=SCOPE_BASE, attrs=["name"])
         res2 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou2,
-                                  scope=SCOPE_BASE, attrs=["name"])
-        print res1[0]["name"][0]
-        print res2[0]["name"][0]
+                                   scope=SCOPE_BASE, attrs=["name"])
+        print(res1[0]["name"][0])
+        print(res2[0]["name"][0])
         self.assertTrue('CNF:%s' % self.ou2 in str(res2[0]["name"][0]))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res1[0].dn))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res2[0].dn))
@@ -393,20 +406,20 @@ objectClass: organizationalUnit
 
         self._net_drs_replicate(DC=self.dnsname_dc1, fromDC=self.dnsname_dc2, forced=True, full_sync=False)
 
-        self.ldb_dc1.rename("<GUID=%s>" % self.ou1, "OU=Test Remote Rename Conflict 3,%s" % self.domain_dn)
+        self.ldb_dc1.rename("<GUID=%s>" % self.ou1, "OU=Test Remote Rename Conflict 3,%s" % self.top_ou)
         # We have to sleep to ensure that the two objects have different timestamps
         time.sleep(1)
-        self.ldb_dc2.rename("<GUID=%s>" % self.ou2, "OU=Test Remote Rename Conflict 3,%s" % self.domain_dn)
+        self.ldb_dc2.rename("<GUID=%s>" % self.ou2, "OU=Test Remote Rename Conflict 3,%s" % self.top_ou)
 
         self._net_drs_replicate(DC=self.dnsname_dc1, fromDC=self.dnsname_dc2, forced=True, full_sync=False)
 
         # Check that DC2 got the DC1 object, and SELF.OU1 was make into conflict
         res1 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou1,
-                                  scope=SCOPE_BASE, attrs=["name"])
+                                   scope=SCOPE_BASE, attrs=["name"])
         res2 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou2,
-                                  scope=SCOPE_BASE, attrs=["name"])
-        print res1[0]["name"][0]
-        print res2[0]["name"][0]
+                                   scope=SCOPE_BASE, attrs=["name"])
+        print(res1[0]["name"][0])
+        print(res2[0]["name"][0])
         self.assertTrue('CNF:%s' % self.ou1 in str(res1[0]["name"][0]))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res1[0].dn))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res2[0].dn))
@@ -426,7 +439,6 @@ objectClass: organizationalUnit
         self._check_deleted(self.ldb_dc2, self.ou1)
         self._check_deleted(self.ldb_dc2, self.ou2)
 
-
     def test_ReplConflictsRenameRemoteWin_with_child(self):
         """Tests that objects created in conflict become conflict DNs"""
         self._disable_inbound_repl(self.dnsname_dc1)
@@ -441,20 +453,20 @@ objectClass: organizationalUnit
 
         self._net_drs_replicate(DC=self.dnsname_dc1, fromDC=self.dnsname_dc2, forced=True, full_sync=False)
 
-        self.ldb_dc1.rename("<GUID=%s>" % self.ou1, "OU=Test Parent Remote Rename Conflict 3,%s" % self.domain_dn)
+        self.ldb_dc1.rename("<GUID=%s>" % self.ou1, "OU=Test Parent Remote Rename Conflict 3,%s" % self.top_ou)
         # We have to sleep to ensure that the two objects have different timestamps
         time.sleep(1)
-        self.ldb_dc2.rename("<GUID=%s>" % self.ou2, "OU=Test Parent Remote Rename Conflict 3,%s" % self.domain_dn)
+        self.ldb_dc2.rename("<GUID=%s>" % self.ou2, "OU=Test Parent Remote Rename Conflict 3,%s" % self.top_ou)
 
         self._net_drs_replicate(DC=self.dnsname_dc1, fromDC=self.dnsname_dc2, forced=True, full_sync=False)
 
         # Check that DC2 got the DC1 object, and SELF.OU1 was make into conflict
         res1 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou1,
-                                  scope=SCOPE_BASE, attrs=["name"])
+                                   scope=SCOPE_BASE, attrs=["name"])
         res2 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou2,
-                                  scope=SCOPE_BASE, attrs=["name"])
-        print res1[0]["name"][0]
-        print res2[0]["name"][0]
+                                   scope=SCOPE_BASE, attrs=["name"])
+        print(res1[0]["name"][0])
+        print(res2[0]["name"][0])
         self.assertTrue('CNF:%s' % self.ou1 in str(res1[0]["name"][0]))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res1[0].dn))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res2[0].dn))
@@ -480,7 +492,6 @@ objectClass: organizationalUnit
         self._check_deleted(self.ldb_dc2, ou1_child)
         self._check_deleted(self.ldb_dc2, ou2_child)
 
-
     def test_ReplConflictsRenameLocalWin(self):
         """Tests that objects created in conflict become conflict DNs"""
         self._disable_inbound_repl(self.dnsname_dc1)
@@ -492,20 +503,20 @@ objectClass: organizationalUnit
 
         self._net_drs_replicate(DC=self.dnsname_dc1, fromDC=self.dnsname_dc2, forced=True, full_sync=False)
 
-        self.ldb_dc2.rename("<GUID=%s>" % self.ou2, "OU=Test Rename Local Conflict 3,%s" % self.domain_dn)
+        self.ldb_dc2.rename("<GUID=%s>" % self.ou2, "OU=Test Rename Local Conflict 3,%s" % self.top_ou)
         # We have to sleep to ensure that the two objects have different timestamps
         time.sleep(1)
-        self.ldb_dc1.rename("<GUID=%s>" % self.ou1, "OU=Test Rename Local Conflict 3,%s" % self.domain_dn)
+        self.ldb_dc1.rename("<GUID=%s>" % self.ou1, "OU=Test Rename Local Conflict 3,%s" % self.top_ou)
 
         self._net_drs_replicate(DC=self.dnsname_dc1, fromDC=self.dnsname_dc2, forced=True, full_sync=False)
 
         # Check that DC2 got the DC1 object, and OU2 was make into conflict
         res1 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou1,
-                                  scope=SCOPE_BASE, attrs=["name"])
+                                   scope=SCOPE_BASE, attrs=["name"])
         res2 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou2,
-                                  scope=SCOPE_BASE, attrs=["name"])
-        print res1[0]["name"][0]
-        print res2[0]["name"][0]
+                                   scope=SCOPE_BASE, attrs=["name"])
+        print(res1[0]["name"][0])
+        print(res2[0]["name"][0])
         self.assertTrue('CNF:%s' % self.ou2 in str(res2[0]["name"][0]))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res1[0].dn))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) not in str(res2[0].dn))
@@ -553,11 +564,11 @@ objectClass: organizationalUnit
 
         # Check that DC2 got the DC1 object, and one or other object was make into conflict
         res1 = self.ldb_dc1.search(base="<GUID=%s>" % ou1_child,
-                                  scope=SCOPE_BASE, attrs=["name"])
+                                   scope=SCOPE_BASE, attrs=["name"])
         res2 = self.ldb_dc1.search(base="<GUID=%s>" % ou2_child,
-                                  scope=SCOPE_BASE, attrs=["name"])
-        print res1[0]["name"][0]
-        print res2[0]["name"][0]
+                                   scope=SCOPE_BASE, attrs=["name"])
+        print(res1[0]["name"][0])
+        print(res2[0]["name"][0])
         self.assertTrue('CNF:%s' % ou1_child in str(res1[0]["name"][0]) or 'CNF:%s' % ou2_child in str(res2[0]["name"][0]))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) in str(res1[0].dn))
         self.assertTrue(self._lost_and_found_dn(self.ldb_dc1, self.domain_dn) in str(res2[0].dn))
@@ -571,7 +582,6 @@ objectClass: organizationalUnit
 
         self._net_drs_replicate(DC=self.dnsname_dc2, fromDC=self.dnsname_dc1, forced=True, full_sync=False)
 
-
         # Check all deleted on DC1
         self._check_deleted(self.ldb_dc1, self.ou1)
         self._check_deleted(self.ldb_dc1, self.ou2)
@@ -603,12 +613,12 @@ objectClass: organizationalUnit
         # replicate them from DC1 to DC2
         self._net_drs_replicate(DC=self.dnsname_dc2, fromDC=self.dnsname_dc1, forced=True, full_sync=False)
 
-        self.ldb_dc1.rename("<GUID=%s>" % ou2_child, "OU=Test Child 3,OU=Original parent 2,%s" % self.domain_dn)
-        self.ldb_dc1.rename("<GUID=%s>" % ou1_child, "OU=Test Child 2,OU=Original parent 2,%s" % self.domain_dn)
-        self.ldb_dc1.rename("<GUID=%s>" % ou2_child, "OU=Test Child,OU=Original parent 2,%s" % self.domain_dn)
-        self.ldb_dc1.rename("<GUID=%s>" % ou3_child, "OU=Test CASE Child,OU=Original parent,%s" % self.domain_dn)
-        self.ldb_dc2.rename("<GUID=%s>" % self.ou2, "OU=Original parent 3,%s" % self.domain_dn)
-        self.ldb_dc2.rename("<GUID=%s>" % self.ou1, "OU=Original parent 2,%s" % self.domain_dn)
+        self.ldb_dc1.rename("<GUID=%s>" % ou2_child, "OU=Test Child 3,OU=Original parent 2,%s" % self.top_ou)
+        self.ldb_dc1.rename("<GUID=%s>" % ou1_child, "OU=Test Child 2,OU=Original parent 2,%s" % self.top_ou)
+        self.ldb_dc1.rename("<GUID=%s>" % ou2_child, "OU=Test Child,OU=Original parent 2,%s" % self.top_ou)
+        self.ldb_dc1.rename("<GUID=%s>" % ou3_child, "OU=Test CASE Child,OU=Original parent,%s" % self.top_ou)
+        self.ldb_dc2.rename("<GUID=%s>" % self.ou2, "OU=Original parent 3,%s" % self.top_ou)
+        self.ldb_dc2.rename("<GUID=%s>" % self.ou1, "OU=Original parent 2,%s" % self.top_ou)
 
         # replicate them from DC1 to DC2
         self._net_drs_replicate(DC=self.dnsname_dc2, fromDC=self.dnsname_dc1, forced=True, full_sync=False)
@@ -618,40 +628,40 @@ objectClass: organizationalUnit
 
         # Check that DC2 got the DC1 object, and the renames are all correct
         res1 = self.ldb_dc2.search(base="<GUID=%s>" % ou1_child,
-                                  scope=SCOPE_BASE, attrs=["name"])
+                                   scope=SCOPE_BASE, attrs=["name"])
         res2 = self.ldb_dc2.search(base="<GUID=%s>" % ou2_child,
-                                  scope=SCOPE_BASE, attrs=["name"])
+                                   scope=SCOPE_BASE, attrs=["name"])
         res3 = self.ldb_dc2.search(base="<GUID=%s>" % ou3_child,
-                                  scope=SCOPE_BASE, attrs=["name"])
-        print res1[0].dn
-        print res2[0].dn
-        print res3[0].dn
-        self.assertEqual('Test Child 2', res1[0]["name"][0])
-        self.assertEqual('Test Child', res2[0]["name"][0])
-        self.assertEqual('Test CASE Child', res3[0]["name"][0])
-        self.assertEqual(str(res1[0].dn), "OU=Test Child 2,OU=Original parent 3,%s" % self.domain_dn)
-        self.assertEqual(str(res2[0].dn), "OU=Test Child,OU=Original parent 3,%s" % self.domain_dn)
-        self.assertEqual(str(res3[0].dn), "OU=Test CASE Child,OU=Original parent 2,%s" % self.domain_dn)
+                                   scope=SCOPE_BASE, attrs=["name"])
+        print(res1[0].dn)
+        print(res2[0].dn)
+        print(res3[0].dn)
+        self.assertEqual('Test Child 2', str(res1[0]["name"][0]))
+        self.assertEqual('Test Child', str(res2[0]["name"][0]))
+        self.assertEqual('Test CASE Child', str(res3[0]["name"][0]))
+        self.assertEqual(str(res1[0].dn), "OU=Test Child 2,OU=Original parent 3,%s" % self.top_ou)
+        self.assertEqual(str(res2[0].dn), "OU=Test Child,OU=Original parent 3,%s" % self.top_ou)
+        self.assertEqual(str(res3[0].dn), "OU=Test CASE Child,OU=Original parent 2,%s" % self.top_ou)
 
         # replicate them from DC2 to DC1
         self._net_drs_replicate(DC=self.dnsname_dc1, fromDC=self.dnsname_dc2, forced=True, full_sync=False)
 
         # Check that DC1 got the DC2 object, and the renames are all correct
         res1 = self.ldb_dc1.search(base="<GUID=%s>" % ou1_child,
-                                  scope=SCOPE_BASE, attrs=["name"])
+                                   scope=SCOPE_BASE, attrs=["name"])
         res2 = self.ldb_dc1.search(base="<GUID=%s>" % ou2_child,
-                                  scope=SCOPE_BASE, attrs=["name"])
+                                   scope=SCOPE_BASE, attrs=["name"])
         res3 = self.ldb_dc1.search(base="<GUID=%s>" % ou3_child,
-                                  scope=SCOPE_BASE, attrs=["name"])
-        print res1[0].dn
-        print res2[0].dn
-        print res3[0].dn
-        self.assertEqual('Test Child 2', res1[0]["name"][0])
-        self.assertEqual('Test Child', res2[0]["name"][0])
-        self.assertEqual('Test CASE Child', res3[0]["name"][0])
-        self.assertEqual(str(res1[0].dn), "OU=Test Child 2,OU=Original parent 3,%s" % self.domain_dn)
-        self.assertEqual(str(res2[0].dn), "OU=Test Child,OU=Original parent 3,%s" % self.domain_dn)
-        self.assertEqual(str(res3[0].dn), "OU=Test CASE Child,OU=Original parent 2,%s" % self.domain_dn)
+                                   scope=SCOPE_BASE, attrs=["name"])
+        print(res1[0].dn)
+        print(res2[0].dn)
+        print(res3[0].dn)
+        self.assertEqual('Test Child 2', str(res1[0]["name"][0]))
+        self.assertEqual('Test Child', str(res2[0]["name"][0]))
+        self.assertEqual('Test CASE Child', str(res3[0]["name"][0]))
+        self.assertEqual(str(res1[0].dn), "OU=Test Child 2,OU=Original parent 3,%s" % self.top_ou)
+        self.assertEqual(str(res2[0].dn), "OU=Test Child,OU=Original parent 3,%s" % self.top_ou)
+        self.assertEqual(str(res3[0].dn), "OU=Test CASE Child,OU=Original parent 2,%s" % self.top_ou)
 
         # Delete all objects by GUID on DC1
 
@@ -663,7 +673,6 @@ objectClass: organizationalUnit
 
         self._net_drs_replicate(DC=self.dnsname_dc2, fromDC=self.dnsname_dc1, forced=True, full_sync=False)
 
-
         # Check all deleted on DC1
         self._check_deleted(self.ldb_dc1, self.ou1)
         self._check_deleted(self.ldb_dc1, self.ou2)
@@ -706,7 +715,7 @@ objectClass: organizationalUnit
 
         # Now pretend that the admin for one DC resolves the problem by
         # re-animating the object...
-        self.reanimate_object(self.ldb_dc1, self.ou1, "OU=Conflict object,%s" % self.domain_dn)
+        self.reanimate_object(self.ldb_dc1, self.ou1, "OU=Conflict object,%s" % self.top_ou)
 
         # ...whereas another admin just creates a user with the same name
         # again on a different DC
@@ -718,11 +727,11 @@ objectClass: organizationalUnit
 
         # Check the latest change won and SELF.OU1 was made into a conflict
         res1 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou1,
-                                  scope=SCOPE_BASE, attrs=["name"])
+                                   scope=SCOPE_BASE, attrs=["name"])
         res2 = self.ldb_dc1.search(base="<GUID=%s>" % self.ou2,
-                                  scope=SCOPE_BASE, attrs=["name"])
-        print res1[0]["name"][0]
-        print res2[0]["name"][0]
+                                   scope=SCOPE_BASE, attrs=["name"])
+        print(res1[0]["name"][0])
+        print(res2[0]["name"][0])
         self.assertTrue('CNF:%s' % self.ou1 in str(res1[0]["name"][0]))
         self.assertFalse('CNF:%s' % self.ou2 in str(res2[0]["name"][0]))
 
@@ -737,4 +746,3 @@ objectClass: organizationalUnit
         # Check deleted on DC2
         self._check_deleted(self.ldb_dc2, self.ou1)
         self._check_deleted(self.ldb_dc2, self.ou2)
-
